@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,14 +32,17 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Available letters for short options:
 
-   a b c d e f i j k l m n p r s t u v w x y z
-   A B C E G H J L Q R W X Y
+   a b c d e f i j k l m n p r t u v x y z
+   A B C E G H J L Q R X Y
 */
 enum option_keys_enum
 {
   /* With short-option version. */
   UI_KEY_GLOBALHDU       = 'g',
   UI_KEY_ONEDASIMAGE     = 'O',
+  UI_KEY_ONEDONSTDOUT    = 's',
+  UI_KEY_WCSFILE         = 'w',
+  UI_KEY_WCSHDU          = 'W',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
@@ -49,6 +52,9 @@ enum option_keys_enum
 void
 ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p);
 
+size_t *
+ui_read_ndim_dsize(char *filename, char *hdu, size_t *ndim);
+
 void
 freeandreport(struct arithmeticparams *p, struct timeval *t1);
 
