@@ -5,7 +5,7 @@ BuildProgram is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -104,6 +104,7 @@ ui_initialize_options(struct buildprogparams *p,
 
 
   /* Set the necessary common parameters structure. */
+  cp->program_struct     = p;
   cp->poptions           = program_options;
   cp->program_name       = PROGRAM_NAME;
   cp->program_exec       = PROGRAM_EXEC;
@@ -123,7 +124,6 @@ ui_initialize_options(struct buildprogparams *p,
         case GAL_OPTIONS_KEY_TYPE:
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_NUMTHREADS:
-        case GAL_OPTIONS_KEY_MINMAPSIZE:
         case GAL_OPTIONS_KEY_TABLEFORMAT:
         case GAL_OPTIONS_KEY_STDINTIMEOUT:
           cp->coptions[i].flags=OPTION_HIDDEN;
@@ -284,6 +284,19 @@ ui_preparations(struct buildprogparams *p)
   if(p->cp.output==NULL)
     p->cp.output=gal_checkset_automatic_output(&p->cp, p->sourceargs->v,
                                                EXEEXT);
+
+  /* Set the C compiler. Later we can add a check to make sure that `cc' is
+     actually in the PATH. */
+  if(p->cc==NULL)
+    {                                        /* No C compiler chosen. */
+      if(p->noenv==0)
+        {
+          p->cc=getenv("CC");                /* First check for `CC'. */
+          if(p->cc==NULL)
+            p->cc=getenv("GCC");             /* Then check for `GCC'. */
+        }
+      if(p->cc==NULL) p->cc="gcc";           /* Default: `gcc'.       */
+    }
 }
 
 
