@@ -32,12 +32,13 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Available letters for short options:
 
-   c e f i j k n p r s u v w x y z
+   f i j k n p r s u v w x y z
    A B C E G H J Q R X Y
 */
 enum option_keys_enum
 {
   /* With short-option version. */
+  UI_KEY_CC             = 'c',
   UI_KEY_INCLUDE        = 'I',
   UI_KEY_LINKDIR        = 'L',
   UI_KEY_LINKLIB        = 'l',
@@ -48,6 +49,7 @@ enum option_keys_enum
   UI_KEY_TAG            = 't',
   UI_KEY_DETELECOMPILED = 'd',
   UI_KEY_LA             = 'a',
+  UI_KEY_NOENV          = 'e',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
