@@ -3,9 +3,9 @@ ConvertType - Convert between various types of files.
 ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -23,6 +23,9 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #ifndef CONVERTT_H
 #define CONVERTT_H
 
+void
+convertt_scale_to_uchar(struct converttparams *p);
+
 void
 convertt(struct converttparams *p);
 
