@@ -376,7 +376,8 @@ ui_read_column(struct convolveparams *p, int i0k1)
 
   /* Read the desired column(s). */
   out=gal_table_read(filename, hdu, lines, column, p->cp.searchin,
-                     p->cp.ignorecase, p->cp.minmapsize, NULL);
+                     p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
+                     NULL);
   gal_list_str_free(lines, 1);
 
   /* Confirm if only one column was read (it is possible to match more than
@@ -384,7 +385,7 @@ ui_read_column(struct convolveparams *p, int i0k1)
   if(out->next!=NULL)
     {
       if(filename)
-        gal_checkset_dataset_name(filename, hdu);
+        source=gal_checkset_dataset_name(filename, hdu);
       else
         source="standard-input";
       error(EXIT_FAILURE, 0, "%s: more than one column in input table mached "
@@ -435,9 +436,13 @@ ui_read_input(struct convolveparams *p)
       {
         p->input=gal_array_read_one_ch_to_type(p->filename, p->cp.hdu, NULL,
                                                INPUT_USE_TYPE,
-                                               p->cp.minmapsize);
+                                               p->cp.minmapsize,
+                                               p->cp.quietmmap);
         p->input->wcs=gal_wcs_read(p->filename, p->cp.hdu, 0, 0,
                                    &p->input->nwcs);
+        p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                                  p->input->dsize,
+                                                  p->input->wcs);
       }
 
   /* The input isn't an image (wasn't read yet), so we'll read it as a
@@ -459,9 +464,15 @@ ui_read_kernel(struct convolveparams *p)
   if( p->kernelname
       && p->input->ndim>1
       && gal_array_name_recognized(p->kernelname)  )
-    p->kernel = gal_array_read_one_ch_to_type(p->kernelname, p->khdu,
-                                              NULL, INPUT_USE_TYPE,
-                                              p->cp.minmapsize);
+    {
+      p->kernel = gal_array_read_one_ch_to_type(p->kernelname, p->khdu,
+                                                NULL, INPUT_USE_TYPE,
+                                                p->cp.minmapsize,
+                                                p->cp.quietmmap);
+      p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
+                                                 p->kernel->dsize,
+                                                 p->kernel->wcs);
+    }
   else
     p->kernel=ui_read_column(p, 1);
 
@@ -492,16 +503,24 @@ ui_preparations(struct convolveparams *p)
   ui_read_input(p);
 
 
-  /* Currently Convolve only works on 1D and 2D datasets. */
-  if(p->input->ndim>2)
+  /* Currently Convolve only works on 1D, 2D and 3D datasets. */
+  if(p->input->ndim>3)
     error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions. Currently "
-          "Convolve only operates on 1D (table column) and 2D (image) "
-          "datasets", p->filename, cp->hdu, p->input->ndim);
+          "Convolve only operates on 1D (table column, spectrum), 2D "
+          "(image), and 3D (data cube) datasets", p->filename, cp->hdu,
+          p->input->ndim);
 
 
   /* Domain-specific checks. */
   if(p->domain==CONVOLVE_DOMAIN_FREQUENCY)
     {
+      /* Check the dimensionality. */
+      if(p->input->ndim!=2)
+        error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions. Frequency "
+              "domain convolution currently only operates on 2D images",
+              p->filename, cp->hdu, p->input->ndim);
+
+      /* Blank values. */
       if( gal_blank_present(p->input, 1) )
         fprintf(stderr, "\n----------------------------------------\n"
                 "######## %s WARNING ########\n"
