@@ -42,7 +42,7 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   f i j k n p t w x y
+   f j k n p t w x y
    B E J Q R W X Y
 */
 enum option_keys_enum
@@ -68,6 +68,7 @@ enum option_keys_enum
   UI_KEY_LOOKBACKTIME        = 'b',
   UI_KEY_CRITICALDENSITY     = 'c',
   UI_KEY_VOLUME              = 'v',
+  UI_KEY_LINEATZ             = 'i',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
