@@ -182,7 +182,6 @@ columns_set_wcs_pointers(struct mkcatalogparams *p, double ***vo,
 
 
 
-
 /******************************************************************/
 /**********       Column definition/allocation      ***************/
 /******************************************************************/
@@ -254,16 +253,10 @@ columns_wcs_preparation(struct mkcatalogparams *p)
 
 
 
-/* Set the necessary parameters for each output column and allocate the
-   space necessary to keep the values. */
-void
-columns_define_alloc(struct mkcatalogparams *p)
+static void
+columns_sanity_check(struct mkcatalogparams *p)
 {
   gal_list_i32_t *colcode;
-  gal_list_str_t *strtmp, *noclumpimg=NULL;
-  int disp_fmt=0, disp_width=0, disp_precision=0;
-  char *name=NULL, *unit=NULL, *ocomment=NULL, *ccomment=NULL;
-  uint8_t otype=GAL_TYPE_INVALID, ctype=GAL_TYPE_INVALID, *oiflag, *ciflag;
 
   /* If there is any columns that need WCS, the input image needs to have a
      WCS in its headers. So before anything, we need to check if a WCS is
@@ -272,6 +265,77 @@ columns_define_alloc(struct mkcatalogparams *p)
      based on it (for units and names). */
   columns_wcs_preparation(p);
 
+  /* Check for dimension-specific columns. */
+  switch(p->objects->ndim)
+    {
+    case 2:
+      for(colcode=p->columnids; colcode!=NULL; colcode=colcode->next)
+        switch(colcode->v)
+          {
+          case UI_KEY_AREAXY:
+          case UI_KEY_GEOAREAXY:
+          case UI_KEY_Z:
+          case UI_KEY_MINZ:
+          case UI_KEY_MAXZ:
+          case UI_KEY_GEOZ:
+          case UI_KEY_CLUMPSZ:
+          case UI_KEY_CLUMPSGEOZ:
+          case UI_KEY_W3:
+          case UI_KEY_GEOW3:
+          case UI_KEY_CLUMPSW3:
+          case UI_KEY_CLUMPSGEOW3:
+            error(EXIT_FAILURE, 0, "%s (hdu %s) is a 2D dataset, so columns "
+                  "relating to a third dimension cannot be requested",
+                  p->objectsfile, p->cp.hdu);
+          }
+      break;
+
+    case 3:
+      for(colcode=p->columnids; colcode!=NULL; colcode=colcode->next)
+        switch(colcode->v)
+          {
+          case UI_KEY_SEMIMAJOR:
+          case UI_KEY_SEMIMINOR:
+          case UI_KEY_AXISRATIO:
+          case UI_KEY_POSITIONANGLE:
+          case UI_KEY_GEOSEMIMAJOR:
+          case UI_KEY_GEOSEMIMINOR:
+          case UI_KEY_GEOAXISRATIO:
+          case UI_KEY_GEOPOSITIONANGLE:
+            error(EXIT_FAILURE, 0, "columns requiring second moment "
+                  "calculations (like semi-major, semi-minor, axis ratio "
+                  "and position angle) are not yet supported for 3D "
+                  "inputs");
+          }
+      break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+            "fix the problem. MakeCatalog should not reach this point "
+            "when the input has %zu dimensions", __func__,
+            PACKAGE_BUGREPORT, p->objects->ndim);
+    }
+}
+
+
+
+
+
+/* Set the necessary parameters for each output column and allocate the
+   space necessary to keep the values. */
+void
+columns_define_alloc(struct mkcatalogparams *p)
+{
+  gal_list_i32_t *colcode;
+  size_t ndim=p->objects->ndim;
+  gal_list_str_t *strtmp, *noclumpimg=NULL;
+  int disp_fmt=0, disp_width=0, disp_precision=0;
+  char *name=NULL, *unit=NULL, *ocomment=NULL, *ccomment=NULL;
+  uint8_t otype=GAL_TYPE_INVALID, ctype=GAL_TYPE_INVALID, *oiflag, *ciflag;
+
+  /* Do a sanity check on the columns given the input dataset. */
+  columns_sanity_check(p);
+
   /* Allocate the array for which intermediate parameters are
      necessary. The basic issue is that higher-level calculations require a
      smaller domain of raw measurements. So to avoid having to calculate
@@ -356,6 +420,19 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_NUM ] = ciflag[ CCOL_NUM ] = 1;
           break;
 
+        case UI_KEY_AREAXY:
+          name           = "AREAXY";
+          unit           = "counter";
+          ocomment       = "Projected valued pixels in first two dimensions.";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_INT32;
+          ctype          = GAL_TYPE_INT32;
+          disp_fmt       = 0;
+          disp_width     = 6;
+          disp_precision = 0;
+          oiflag[ OCOL_NUMXY ] = ciflag[ CCOL_NUMXY ] = 1;
+          break;
+
         case UI_KEY_CLUMPSAREA:
           name           = "AREA_CLUMPS";
           unit           = "counter";
@@ -395,6 +472,19 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
+        case UI_KEY_GEOAREAXY:
+          name           = "AREA_FULL_XY";
+          unit           = "counter";
+          ocomment       = "Project number in first two dimensions.";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_INT32;
+          ctype          = GAL_TYPE_INT32;
+          disp_fmt       = 0;
+          disp_width     = 6;
+          disp_precision = 0;
+          oiflag[ OCOL_NUMALLXY ] = ciflag[ CCOL_NUMALLXY ] = 1;
+          break;
+
         case UI_KEY_X:
           name           = "X";
           unit           = "pixel";
@@ -427,6 +517,20 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_NUMWHT ] = ciflag[ CCOL_NUMWHT ] = 1;
           break;
 
+        case UI_KEY_Z:
+          name           = "Z";
+          unit           = "pixel";
+          ocomment       = "Flux weighted center (FITS axis 3).";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_FLOAT32;
+          ctype          = GAL_TYPE_FLOAT32;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 10;
+          disp_precision = 3;
+          oiflag[ OCOL_VZ ] = 1;
+          ciflag[ CCOL_VZ ] = 1;
+          break;
+
         case UI_KEY_GEOX:
           name           = "GEO_X";
           unit           = "pixel";
@@ -455,6 +559,20 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
+        case UI_KEY_GEOZ:
+          name           = "GEO_Z";
+          unit           = "pixel";
+          ocomment       = "Geometric center (FITS axis 3).";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_FLOAT32;
+          ctype          = GAL_TYPE_FLOAT32;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 10;
+          disp_precision = 3;
+          oiflag[ OCOL_GZ ] = 1;
+          ciflag[ CCOL_GZ ] = 1;
+          break;
+
         case UI_KEY_CLUMPSX:
           name           = "CLUMPS_X";
           unit           = "pixel";
@@ -487,6 +605,19 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_C_NUMWHT ] = 1;
           break;
 
+        case UI_KEY_CLUMPSZ:
+          name           = "CLUMPS_Z";
+          unit           = "pixel";
+          ocomment       = "Flux weighted center of clumps (FITS axis 3).";
+          ccomment       = NULL;
+          otype          = GAL_TYPE_FLOAT32;
+          ctype          = GAL_TYPE_INVALID;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 10;
+          disp_precision = 3;
+          oiflag[ OCOL_C_VZ ] = 1;
+          break;
+
         case UI_KEY_CLUMPSGEOX:
           name           = "CLUMPS_GEO_X";
           unit           = "pixel";
@@ -515,6 +646,18 @@ columns_define_alloc(struct mkcatalogparams *p)
           oiflag[ OCOL_C_NUMALL ] = 1;
           break;
 
+        case UI_KEY_CLUMPSGEOZ:
+          name           = "CLUMPS_GEO_Z";
+          unit           = "pixel";
+          ocomment       = "Geometric center of clumps (FITS axis 3).";
+          ccomment       = NULL;
+          otype          = GAL_TYPE_FLOAT32;
+          ctype          = GAL_TYPE_INVALID;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 10;
+          disp_precision = 3;
+          oiflag[ OCOL_C_GZ ] = 1;
+
         case UI_KEY_MINX:
           name           = "MIN_X";
           unit           = "pixel";
@@ -526,6 +669,7 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 10;
           disp_precision = 0;
           ciflag[ CCOL_MINX ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
         case UI_KEY_MAXX:
@@ -539,6 +683,7 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 10;
           disp_precision = 0;
           ciflag[ CCOL_MAXX ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
         case UI_KEY_MINY:
@@ -552,6 +697,7 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 10;
           disp_precision = 0;
           ciflag[ CCOL_MINY ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
         case UI_KEY_MAXY:
@@ -565,6 +711,35 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 10;
           disp_precision = 0;
           ciflag[ CCOL_MAXY ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          break;
+
+        case UI_KEY_MINZ:
+          name           = "MIN_Z";
+          unit           = "pixel";
+          ocomment       = "Minimum Z axis pixel position.";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_UINT32;
+          ctype          = GAL_TYPE_UINT32;
+          disp_fmt       = 0;
+          disp_width     = 10;
+          disp_precision = 0;
+          ciflag[ CCOL_MINZ ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          break;
+
+        case UI_KEY_MAXZ:
+          name           = "MAX_Z";
+          unit           = "pixel";
+          ocomment       = "Maximum Z axis pixel position.";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_UINT32;
+          ctype          = GAL_TYPE_UINT32;
+          disp_fmt       = 0;
+          disp_width     = 10;
+          disp_precision = 0;
+          ciflag[ CCOL_MAXZ ] = 1;
+          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
         case UI_KEY_W1:
@@ -578,12 +753,17 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_radec(p);
-          oiflag[ OCOL_VX     ] = ciflag[ CCOL_VX     ] = 1;
-          oiflag[ OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
-          oiflag[ OCOL_VY     ] = ciflag[ CCOL_VY     ] = 1;
-          oiflag[ OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
-          oiflag[ OCOL_SUMWHT ] = ciflag[ CCOL_SUMWHT ] = 1;
-          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          oiflag[     OCOL_VX     ] = ciflag[ CCOL_VX     ] = 1;
+          oiflag[     OCOL_VY     ] = ciflag[ CCOL_VY     ] = 1;
+          oiflag[     OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
+          oiflag[     OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[     OCOL_SUMWHT ] = ciflag[ CCOL_SUMWHT ] = 1;
+          oiflag[     OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          if(ndim==3)
+            {
+              oiflag[ OCOL_VZ ] = ciflag[ CCOL_VZ ] = 1;
+              oiflag[ OCOL_GZ ] = ciflag[ CCOL_GZ ] = 1;
+            }
           break;
 
         case UI_KEY_W2:
@@ -597,10 +777,36 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_radec(p);
+          oiflag[     OCOL_VX     ] = ciflag[ CCOL_VX     ] = 1;
+          oiflag[     OCOL_VY     ] = ciflag[ CCOL_VY     ] = 1;
+          oiflag[     OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
+          oiflag[     OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[     OCOL_SUMWHT ] = ciflag[ CCOL_SUMWHT ] = 1;
+          oiflag[     OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          if(ndim==3)
+            {
+              oiflag[ OCOL_VZ     ] = ciflag[ CCOL_VZ     ] = 1;
+              oiflag[ OCOL_GZ     ] = ciflag[ CCOL_GZ     ] = 1;
+            }
+          break;
+
+        case UI_KEY_W3:
+          name           = p->ctype[2];
+          unit           = p->objects->wcs->cunit[2];
+          ocomment       = "Flux weighted center (WCS axis 3).";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_FLOAT64;
+          ctype          = GAL_TYPE_FLOAT64;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 13;
+          disp_precision = 7;
+          columns_alloc_radec(p);
           oiflag[ OCOL_VX     ] = ciflag[ CCOL_VX     ] = 1;
-          oiflag[ OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
           oiflag[ OCOL_VY     ] = ciflag[ CCOL_VY     ] = 1;
+          oiflag[ OCOL_VZ     ] = ciflag[ CCOL_VZ     ] = 1;
+          oiflag[ OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
           oiflag[ OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[ OCOL_GZ     ] = ciflag[ CCOL_GZ     ] = 1;
           oiflag[ OCOL_SUMWHT ] = ciflag[ CCOL_SUMWHT ] = 1;
           oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
@@ -616,9 +822,11 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_georadec(p);
-          oiflag[ OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
-          oiflag[ OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
-          oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          oiflag[   OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
+          oiflag[   OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[   OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          if(ndim==3)
+            oiflag[ OCOL_GZ     ] = ciflag[ CCOL_GZ     ] = 1;
           break;
 
         case UI_KEY_GEOW2:
@@ -632,8 +840,27 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_georadec(p);
+          oiflag[   OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
+          oiflag[   OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[   OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
+          if(ndim==3)
+            oiflag[ OCOL_GZ     ] = ciflag[ CCOL_GZ     ] = 1;
+          break;
+
+        case UI_KEY_GEOW3:
+          name           = gal_checkset_malloc_cat("GEO_", p->ctype[2]);
+          unit           = p->objects->wcs->cunit[2];
+          ocomment       = "Geometric center (WCS axis 3).";
+          ccomment       = ocomment;
+          otype          = GAL_TYPE_FLOAT64;
+          ctype          = GAL_TYPE_FLOAT64;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 13;
+          disp_precision = 7;
+          columns_alloc_georadec(p);
           oiflag[ OCOL_GX     ] = ciflag[ CCOL_GX     ] = 1;
           oiflag[ OCOL_GY     ] = ciflag[ CCOL_GY     ] = 1;
+          oiflag[ OCOL_GZ     ] = ciflag[ CCOL_GZ     ] = 1;
           oiflag[ OCOL_NUMALL ] = ciflag[ CCOL_NUMALL ] = 1;
           break;
 
@@ -648,12 +875,17 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_clumpsradec(p);
-          oiflag[ OCOL_C_VX     ] = 1;
-          oiflag[ OCOL_C_VY     ] = 1;
-          oiflag[ OCOL_C_GX     ] = 1;
-          oiflag[ OCOL_C_GY     ] = 1;
-          oiflag[ OCOL_C_SUMWHT ] = 1;
-          oiflag[ OCOL_C_NUMALL ] = 1;
+          oiflag[     OCOL_C_VX     ] = 1;
+          oiflag[     OCOL_C_VY     ] = 1;
+          oiflag[     OCOL_C_GX     ] = 1;
+          oiflag[     OCOL_C_GY     ] = 1;
+          oiflag[     OCOL_C_SUMWHT ] = 1;
+          oiflag[     OCOL_C_NUMALL ] = 1;
+          if(ndim==3)
+            {
+              oiflag[ OCOL_C_VZ     ] = 1;
+              oiflag[ OCOL_C_GZ     ] = 1;
+            }
           break;
 
         case UI_KEY_CLUMPSW2:
@@ -667,12 +899,41 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 15;
           disp_precision = 7;
           columns_alloc_clumpsradec(p);
+          oiflag[     OCOL_C_VX     ] = 1;
+          oiflag[     OCOL_C_VY     ] = 1;
+          oiflag[     OCOL_C_GX     ] = 1;
+          oiflag[     OCOL_C_GY     ] = 1;
+          oiflag[     OCOL_C_SUMWHT ] = 1;
+          oiflag[     OCOL_C_NUMALL ] = 1;
+          if(ndim==3)
+            {
+              oiflag[ OCOL_C_VZ     ] = 1;
+              oiflag[ OCOL_C_GZ     ] = 1;
+            }
+          break;
+
+        case UI_KEY_CLUMPSW3:
+          name           = gal_checkset_malloc_cat("CLUMPS_", p->ctype[2]);
+          unit           = p->objects->wcs->cunit[2];
+          ocomment       = "Flux.wht center of all clumps (WCS axis 3).";
+          ccomment       = NULL;
+          otype          = GAL_TYPE_FLOAT64;
+          ctype          = GAL_TYPE_INVALID;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 15;
+          disp_precision = 7;
+          columns_alloc_clumpsradec(p);
           oiflag[ OCOL_C_VX     ] = 1;
           oiflag[ OCOL_C_VY     ] = 1;
+          oiflag[ OCOL_C_VZ     ] = 1;
           oiflag[ OCOL_C_GX     ] = 1;
           oiflag[ OCOL_C_GY     ] = 1;
+          oiflag[ OCOL_C_GZ     ] = 1;
           oiflag[ OCOL_C_SUMWHT ] = 1;
           oiflag[ OCOL_C_NUMALL ] = 1;
+          if(ndim==3)
+            {
+            }
           break;
 
         case UI_KEY_CLUMPSGEOW1:
@@ -686,9 +947,11 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_clumpsgeoradec(p);
-          oiflag[ OCOL_C_GX     ] = 1;
-          oiflag[ OCOL_C_GY     ] = 1;
-          oiflag[ OCOL_C_NUMALL ] = 1;
+          oiflag[   OCOL_C_GX     ] = 1;
+          oiflag[   OCOL_C_GY     ] = 1;
+          oiflag[   OCOL_C_NUMALL ] = 1;
+          if(ndim==3)
+            oiflag[ OCOL_C_GZ     ] = 1;
           break;
 
         case UI_KEY_CLUMPSGEOW2:
@@ -702,8 +965,27 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 13;
           disp_precision = 7;
           columns_alloc_clumpsgeoradec(p);
+          oiflag[   OCOL_C_GX     ] = 1;
+          oiflag[   OCOL_C_GY     ] = 1;
+          oiflag[   OCOL_C_NUMALL ] = 1;
+          if(ndim==3)
+            oiflag[ OCOL_C_GZ     ] = 1;
+          break;
+
+        case UI_KEY_CLUMPSGEOW3:
+          name           = gal_checkset_malloc_cat("CLUMPS_GEO", p->ctype[2]);
+          unit           = p->objects->wcs->cunit[2];
+          ocomment       = "Geometric center of all clumps (WCS axis 3).";
+          ccomment       = NULL;
+          otype          = GAL_TYPE_FLOAT64;
+          ctype          = GAL_TYPE_INVALID;
+          disp_fmt       = GAL_TABLE_DISPLAY_FMT_FLOAT;
+          disp_width     = 13;
+          disp_precision = 7;
+          columns_alloc_clumpsgeoradec(p);
           oiflag[ OCOL_C_GX     ] = 1;
           oiflag[ OCOL_C_GY     ] = 1;
+          oiflag[ OCOL_C_GZ     ] = 1;
           oiflag[ OCOL_C_NUMALL ] = 1;
           break;
 
@@ -1445,8 +1727,9 @@ columns_clump_brightness(double *ci)
 
 /* Measure the minimum and maximum positions. */
 static uint32_t
-columns_xy_extrema(struct mkcatalog_passparams *pp, size_t *coord, int key)
+columns_xy_extrema(struct mkcatalog_passparams *pp, double *oi, size_t *coord, int key)
 {
+  size_t ndim=pp->tile->ndim;
   gal_data_t *tile=pp->tile, *block=tile->block;
 
   /* We only want to do the coordinate estimation once: in `columns_fill',
@@ -1460,18 +1743,24 @@ columns_xy_extrema(struct mkcatalog_passparams *pp, size_t *coord, int key)
                                  block->ndim, block->dsize, coord);
 
   /* Return the proper value: note that `coord' is in C standard: starting
-     from the slowest dimension and counting from zero.*/
-  switch(key)
-    {
-    case UI_KEY_MINX: return coord[1] + 1;              break;
-    case UI_KEY_MAXX: return coord[1] + tile->dsize[1]; break;
-    case UI_KEY_MINY: return coord[0] + 1;              break;
-    case UI_KEY_MAXY: return coord[0] + tile->dsize[0]; break;
-    default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
-            "problem. The value %d is not a recognized value", __func__,
-            PACKAGE_BUGREPORT, key);
-    }
+     from the slowest dimension and counting from zero. */
+  if(oi[OCOL_NUMALL])
+    switch(key)
+      {
+      case UI_KEY_MINX: return coord[ndim-1] + 1;                   break;
+      case UI_KEY_MAXX: return coord[ndim-1] + tile->dsize[ndim-1]; break;
+      case UI_KEY_MINY: return coord[ndim-2] + 1;                   break;
+      case UI_KEY_MAXY: return coord[ndim-2] + tile->dsize[ndim-2]; break;
+      case UI_KEY_MINZ: return coord[ndim-3] + 1;                   break;
+      case UI_KEY_MAXZ: return coord[ndim-3] + tile->dsize[ndim-3]; break;
+      default:
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+              "problem. The value %d is not a recognized value", __func__,
+              PACKAGE_BUGREPORT, key);
+      }
+  else
+    return 0;
+
 
   /* Control should not reach here. */
   error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix the "
@@ -1538,12 +1827,20 @@ columns_fill(struct mkcatalog_passparams *pp)
   void *colarr;
   gal_data_t *column;
   double *ci, *oi=pp->oi;
-  size_t coord[2]={GAL_BLANK_SIZE_T, GAL_BLANK_SIZE_T};
+  size_t coord[3]={GAL_BLANK_SIZE_T, GAL_BLANK_SIZE_T, GAL_BLANK_SIZE_T};
 
-  size_t sr=pp->clumpstartindex, cind, coind;
-  size_t oind=pp->object-1; /* IDs start from 1, indexs from 0. */
+  size_t i, cind, coind, sr=pp->clumpstartindex, oind=GAL_BLANK_SIZE_T;
   double **vo=NULL, **vc=NULL, **go=NULL, **gc=NULL, **vcc=NULL, **gcc=NULL;
 
+  /* Find the object's index in final catalog. */
+  if(p->outlabs)
+    {
+      for(i=0;i<p->numobjects;++i)
+        if(p->outlabs[i]==pp->object)
+          { oind=i; break; }
+    }
+  else oind=pp->object-1;
+
   /* If a WCS column is requested (check will be done inside the function),
      then set the pointers. */
   columns_set_wcs_pointers(p, &vo, &vc, &go, &gc, &vcc, &gcc);
@@ -1575,6 +1872,10 @@ columns_fill(struct mkcatalog_passparams *pp)
           ((int32_t *)colarr)[oind] = oi[OCOL_NUM];
           break;
 
+        case UI_KEY_AREAXY:
+          ((int32_t *)colarr)[oind] = oi[OCOL_NUMXY];
+          break;
+
         case UI_KEY_CLUMPSAREA:
           ((int32_t *)colarr)[oind] = oi[OCOL_C_NUM];
           break;
@@ -1587,6 +1888,10 @@ columns_fill(struct mkcatalog_passparams *pp)
           ((int32_t *)colarr)[oind] = oi[OCOL_NUMALL];
           break;
 
+        case UI_KEY_GEOAREAXY:
+          ((int32_t *)colarr)[oind] = oi[OCOL_NUMALLXY];
+          break;
+
         case UI_KEY_X:
           ((float *)colarr)[oind] = POS_V_G(oi, OCOL_SUMWHT, OCOL_NUMWHT,
                                             OCOL_VX, OCOL_GX);
@@ -1597,6 +1902,11 @@ columns_fill(struct mkcatalog_passparams *pp)
                                             OCOL_VY, OCOL_GY);
           break;
 
+        case UI_KEY_Z:
+          ((float *)colarr)[oind] = POS_V_G(oi, OCOL_SUMWHT, OCOL_NUMALL,
+                                            OCOL_VZ, OCOL_GZ);
+          break;
+
         case UI_KEY_GEOX:
           ((float *)colarr)[oind] = MKC_RATIO( oi[OCOL_GX], oi[OCOL_NUMALL] );
           break;
@@ -1605,6 +1915,10 @@ columns_fill(struct mkcatalog_passparams *pp)
           ((float *)colarr)[oind] = MKC_RATIO( oi[OCOL_GY], oi[OCOL_NUMALL] );
           break;
 
+        case UI_KEY_GEOZ:
+          ((float *)colarr)[oind] = MKC_RATIO( oi[OCOL_GZ], oi[OCOL_NUMALL] );
+          break;
+
         case UI_KEY_CLUMPSX:
           ((float *)colarr)[oind] = POS_V_G(oi, OCOL_C_SUMWHT, OCOL_C_NUMWHT,
                                             OCOL_C_VX, OCOL_C_GX);
@@ -1615,6 +1929,11 @@ columns_fill(struct mkcatalog_passparams *pp)
                                             OCOL_C_VY, OCOL_C_GY);
           break;
 
+        case UI_KEY_CLUMPSZ:
+          ((float *)colarr)[oind] = POS_V_G(oi, OCOL_C_SUMWHT, OCOL_C_NUMALL,
+                                            OCOL_C_VZ, OCOL_C_GZ);
+          break;
+
         case UI_KEY_CLUMPSGEOX:
           ((float *)colarr)[oind] = MKC_RATIO( oi[OCOL_C_GX],
                                                oi[OCOL_C_NUMALL] );
@@ -1625,39 +1944,59 @@ columns_fill(struct mkcatalog_passparams *pp)
                                                oi[OCOL_C_NUMALL] );
           break;
 
+        case UI_KEY_CLUMPSGEOZ:
+          ((float *)colarr)[oind] = MKC_RATIO( oi[OCOL_C_GZ],
+                                               oi[OCOL_C_NUMALL] );
+
         case UI_KEY_MINX:
         case UI_KEY_MAXX:
         case UI_KEY_MINY:
         case UI_KEY_MAXY:
-          ((uint32_t *)colarr)[oind]=columns_xy_extrema(pp, coord, key);
+        case UI_KEY_MINZ:
+        case UI_KEY_MAXZ:
+          ((uint32_t *)colarr)[oind]=columns_xy_extrema(pp, oi, coord, key);
           break;
 
         case UI_KEY_W1:
         case UI_KEY_W2:
+        case UI_KEY_W3:
           vo[0][oind] = POS_V_G(oi, OCOL_SUMWHT, OCOL_NUMALL, OCOL_VX,
                                 OCOL_GX);
           vo[1][oind] = POS_V_G(oi, OCOL_SUMWHT, OCOL_NUMALL, OCOL_VY,
                                 OCOL_GY);
+          if(p->objects->ndim==3)
+            vo[2][oind] = POS_V_G(oi, OCOL_SUMWHT, OCOL_NUMALL, OCOL_VZ,
+                                  OCOL_GZ);
           break;
 
         case UI_KEY_GEOW1:
         case UI_KEY_GEOW2:
+        case UI_KEY_GEOW3:
           go[0][oind] = MKC_RATIO( oi[OCOL_GX], oi[OCOL_NUMALL] );
           go[1][oind] = MKC_RATIO( oi[OCOL_GY], oi[OCOL_NUMALL] );
+          if(p->objects->ndim==3)
+            go[2][oind] = MKC_RATIO( oi[OCOL_GZ], oi[OCOL_NUMALL] );
           break;
 
         case UI_KEY_CLUMPSW1:
         case UI_KEY_CLUMPSW2:
+        case UI_KEY_CLUMPSW3:
           vcc[0][oind] = POS_V_G(oi, OCOL_C_SUMWHT, OCOL_C_NUMALL, OCOL_C_VX,
                                  OCOL_C_GX);
           vcc[1][oind] = POS_V_G(oi, OCOL_C_SUMWHT, OCOL_C_NUMALL, OCOL_C_VY,
                                  OCOL_C_GY);
+          if(p->objects->ndim==3)
+            vcc[2][oind] = POS_V_G(oi, OCOL_C_SUMWHT, OCOL_C_NUMALL,
+                                   OCOL_C_VZ, OCOL_C_GZ);
           break;
 
         case UI_KEY_CLUMPSGEOW1:
         case UI_KEY_CLUMPSGEOW2:
+        case UI_KEY_CLUMPSGEOW3:
           gcc[0][oind] = MKC_RATIO( oi[OCOL_C_GX], oi[OCOL_C_NUMALL] );
           gcc[1][oind] = MKC_RATIO( oi[OCOL_C_GY], oi[OCOL_C_NUMALL] );
+          if(p->objects->ndim==3)
+            gcc[2][oind] = MKC_RATIO( oi[OCOL_C_GZ], oi[OCOL_C_NUMALL] );
           break;
 
         case UI_KEY_BRIGHTNESS:
@@ -1735,12 +2074,12 @@ columns_fill(struct mkcatalog_passparams *pp)
           break;
 
         case UI_KEY_SKY:
-          ((float *)colarr)[oind] = MKC_RATIO(oi[OCOL_SUMSKY], oi[OCOL_NUM]);
+          ((float *)colarr)[oind] = MKC_RATIO(oi[OCOL_SUMSKY], oi[OCOL_NUMSKY]);
           break;
 
         case UI_KEY_STD:
           ((float *)colarr)[oind] = sqrt( MKC_RATIO( oi[ OCOL_SUMVAR ],
-                                                     oi[ OCOL_NUM    ]) );
+                                                     oi[ OCOL_NUMVAR ]) );
           break;
 
         case UI_KEY_SEMIMAJOR:
@@ -1817,6 +2156,10 @@ columns_fill(struct mkcatalog_passparams *pp)
             ((int32_t *)colarr)[cind]=ci[CCOL_NUM];
             break;
 
+          case UI_KEY_AREAXY:
+            ((int32_t *)colarr)[cind]=ci[CCOL_NUMXY];
+            break;
+
           case UI_KEY_WEIGHTAREA:
             ((int32_t *)colarr)[cind]=ci[CCOL_NUMWHT];
             break;
@@ -1825,6 +2168,10 @@ columns_fill(struct mkcatalog_passparams *pp)
             ((int32_t *)colarr)[cind]=ci[CCOL_NUMALL];
             break;
 
+          case UI_KEY_GEOAREAXY:
+            ((int32_t *)colarr)[cind]=ci[CCOL_NUMALLXY];
+            break;
+
           case UI_KEY_X:
             ((float *)colarr)[cind] = POS_V_G(ci, CCOL_SUMWHT, CCOL_NUMALL,
                                               CCOL_VX, CCOL_GX);
@@ -1835,6 +2182,11 @@ columns_fill(struct mkcatalog_passparams *pp)
                                               CCOL_VY, CCOL_GY);
             break;
 
+          case UI_KEY_Z:
+            ((float *)colarr)[cind] = POS_V_G(ci, CCOL_SUMWHT, CCOL_NUMALL,
+                                              CCOL_VZ, CCOL_GZ);
+            break;
+
           case UI_KEY_GEOX:
             ((float *)colarr)[cind] = MKC_RATIO( ci[CCOL_GX],
                                                  ci[CCOL_NUMALL] );
@@ -1845,34 +2197,53 @@ columns_fill(struct mkcatalog_passparams *pp)
                                                  ci[CCOL_NUMALL] );
             break;
 
-         case UI_KEY_MINX:
+          case UI_KEY_GEOZ:
+            ((float *)colarr)[cind] = MKC_RATIO( ci[CCOL_GZ],
+                                                 ci[CCOL_NUMALL] );
+
+          case UI_KEY_MINX:
             ((uint32_t *)colarr)[cind] = ci[CCOL_MINX];
             break;
 
-         case UI_KEY_MAXX:
+          case UI_KEY_MAXX:
             ((uint32_t *)colarr)[cind] = ci[CCOL_MAXX];
             break;
 
-         case UI_KEY_MINY:
+          case UI_KEY_MINY:
             ((uint32_t *)colarr)[cind] = ci[CCOL_MINY];
             break;
 
-         case UI_KEY_MAXY:
+          case UI_KEY_MAXY:
             ((uint32_t *)colarr)[cind] = ci[CCOL_MAXY];
             break;
 
+          case UI_KEY_MINZ:
+            ((uint32_t *)colarr)[cind] = ci[CCOL_MINZ];
+            break;
+
+          case UI_KEY_MAXZ:
+            ((uint32_t *)colarr)[cind] = ci[CCOL_MAXZ];
+            break;
+
           case UI_KEY_W1:
           case UI_KEY_W2:
+          case UI_KEY_W3:
             vc[0][cind] = POS_V_G(ci, CCOL_SUMWHT, CCOL_NUMALL, CCOL_VX,
                                   CCOL_GX);
             vc[1][cind] = POS_V_G(ci, CCOL_SUMWHT, CCOL_NUMALL, CCOL_VY,
                                   CCOL_GY);
+            if(p->objects->ndim==3)
+              vc[2][cind] = POS_V_G(ci, CCOL_SUMWHT, CCOL_NUMALL, CCOL_VZ,
+                                    CCOL_GZ);
             break;
 
           case UI_KEY_GEOW1:
           case UI_KEY_GEOW2:
+          case UI_KEY_GEOW3:
             gc[0][cind] = MKC_RATIO( ci[CCOL_GX], ci[CCOL_NUMALL] );
             gc[1][cind] = MKC_RATIO( ci[CCOL_GY], ci[CCOL_NUMALL] );
+            if(p->objects->ndim==3)
+              gc[2][cind] = MKC_RATIO( ci[CCOL_GZ], ci[CCOL_NUMALL] );
             break;
 
           case UI_KEY_BRIGHTNESS:
@@ -1952,12 +2323,12 @@ columns_fill(struct mkcatalog_passparams *pp)
 
           case UI_KEY_SKY:
             ((float *)colarr)[cind] = MKC_RATIO( ci[ CCOL_SUMSKY],
-                                                 ci[ CCOL_NUM] );
+                                                 ci[ CCOL_NUMSKY] );
             break;
 
           case UI_KEY_STD:
             ((float *)colarr)[cind] = sqrt( MKC_RATIO( ci[ CCOL_SUMVAR ],
-                                                       ci[ CCOL_NUM    ] ));
+                                                       ci[ CCOL_NUMVAR ] ));
             break;
 
           case UI_KEY_SEMIMAJOR:
