@@ -612,6 +612,144 @@ ui_num_clumps(struct mkcatalogparams *p)
 
 
 
+/* To make the catalog processing more scalable (and later allow for
+   over-lappping regions), we will define a tile for each object. */
+static void
+ui_one_tile_per_object_correct_numobjects(struct mkcatalogparams *p)
+{
+  size_t ndim=p->objects->ndim;
+
+  uint8_t *rarray=NULL;
+  int32_t *l, *lf, *start;
+  gal_data_t *rowsremove=NULL;
+  size_t i, j, d, no, *min, *max, exists, width=2*ndim;
+  size_t *minmax=gal_pointer_allocate(GAL_TYPE_SIZE_T,
+                                      width*p->numobjects, 0, __func__,
+                                      "minmax");
+  size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                                      "coord");
+
+  /* Initialize the minimum and maximum position for each tile/object. So,
+     we'll initialize the minimum coordinates to the maximum possible
+     `size_t' value (in `GAL_BLANK_SIZE_T') and the maximums to zero. */
+  for(i=0;i<p->numobjects;++i)
+    for(d=0;d<ndim;++d)
+      {
+        minmax[ i * width +        d ] = GAL_BLANK_SIZE_T; /* Minimum. */
+        minmax[ i * width + ndim + d ] = 0;                /* Maximum. */
+      }
+
+  /* Go over the objects label image and correct the minimum and maximum
+     coordinates. */
+  start=p->objects->array;
+  lf=(l=p->objects->array)+p->objects->size;
+  do
+    if(*l>0)
+      {
+        /* Get the coordinates of this pixel. */
+        gal_dimension_index_to_coord(l-start, ndim, p->objects->dsize,
+                                     coord);
+
+        /* Check to see this coordinate is the smallest/largest found so
+           far for this label. Note that labels start from 1, while indexs
+           here start from zero. */
+        min = &minmax[ (*l-1) * width        ];
+        max = &minmax[ (*l-1) * width + ndim ];
+        for(d=0;d<ndim;++d)
+          {
+            if( coord[d] < min[d] ) min[d] = coord[d];
+            if( coord[d] > max[d] ) max[d] = coord[d];
+          }
+      }
+  while(++l<lf);
+
+  /* If a label doesn't exist in the image, then write over it and define
+     the unique labels to use for the next steps. To over-write, we have
+     two counters: `i' (for the position in the input array) and `no' (or
+     `num-objects' for the counter in the output table). In the end, `no'
+     counts the total number of unique labels in the input. */
+  no=0;
+  for(i=0;i<p->numobjects;++i)
+    {
+      /* Make sure a pixel with this label exists in all dimensions. */
+      exists=0;
+      for(d=0;d<ndim;++d)
+        if ( minmax[ i * width + d ] == GAL_BLANK_SIZE_T
+             && minmax[ i * width + ndim + d ] == 0 )
+          {
+            /* When the object doesn't exist, but the user wants a row
+               anyway, make all the minimums and maximums of all
+               coordinates 0, note that the maximum is already zero. */
+            if(p->inbetweenints)
+              minmax[ i * width + d ] = 0;
+          }
+        else
+          {
+            /* Write over the blank elements when necessary
+               (i!=j). When i==j, then these statements are
+               redundant. */
+            minmax[no*width+d]=minmax[i*width+d];
+            minmax[no*width+ndim+d]=minmax[i*width+ndim+d];
+
+            /* Set the checked flag. */
+            exists=1;
+          }
+
+      /* If it does (or if the user wants to keep all integers), then
+         increment the output counter.*/
+      if(p->inbetweenints || exists) ++no;
+      else
+        {
+          /* If `rarray' isn't defined yet, define it. */
+          if(rarray==NULL)
+            {
+              /* Note that by initializing with zeros, all (the possibly
+                 existing) previous rows that shouldn't be removed are
+                 flagged as zero in this array. */
+              rowsremove=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1,
+                                        &p->numobjects, NULL, 1,
+                                        p->cp.minmapsize, p->cp.quietmmap,
+                                        NULL, NULL, NULL);
+              rarray=rowsremove->array;
+            }
+          rarray[i]=1;
+        }
+    }
+
+  /* If `rarray!=NULL', then there are elements to remove and we need to
+     make some modifications/corrections. */
+  if(rarray)
+    {
+      /* Build an array to keep the real ID of each tile. */
+      j=0;
+      p->outlabs=gal_pointer_allocate(GAL_TYPE_INT32, no, 0, __func__,
+                                      "p->outlabs");
+      for(i=0;i<p->numobjects;++i) if(rarray[i]==0) p->outlabs[j++]=i+1;
+
+      /* Correct numobjects and clean up. */
+      p->numobjects=no;
+      gal_data_free(rowsremove);
+    }
+
+  /* For a check.
+  for(i=0;i<p->numobjects;++i)
+    printf("%zu: (%zu, %zu) --> (%zu, %zu)\n",
+           p->outlabs ? p->outlabs[i] : i+1, minmax[i*width],
+           minmax[i*width+1], minmax[i*width+2], minmax[i*width+3]);
+  */
+
+  /* Make the tiles. */
+  p->tiles=gal_tile_series_from_minmax(p->objects, minmax, p->numobjects);
+
+  /* Clean up. */
+  free(coord);
+  free(minmax);
+}
+
+
+
+
+
 /* The only mandatory input is the objects image, so first read that and
    make sure its type is correct. */
 static void
@@ -634,11 +772,17 @@ ui_read_labels(struct mkcatalogparams *p)
   p->objects=gal_data_copy_to_new_type_free(p->objects, GAL_TYPE_INT32);
 
 
-  /* Currently MakeCatalog is only implemented for 2D images. */
-  if(p->objects->ndim!=2)
+  /* Currently MakeCatalog is only implemented for 2D images or 3D cubes. */
+  if(p->objects->ndim!=2 && p->objects->ndim!=3)
     error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions, MakeCatalog "
-          "currently only supports 2D inputs", p->objectsfile, p->cp.hdu,
-          p->objects->ndim);
+          "currently only supports 2D or 3D datasets", p->objectsfile,
+          p->cp.hdu, p->objects->ndim);
+
+  /* Make sure the `--spectrum' option is not given on a 2D image.  */
+  if(p->spectrum && p->objects->ndim!=3)
+    error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions, but `--spectrum' "
+          "is currently only defined on 3D datasets", p->objectsfile,
+          p->cp.hdu, p->objects->ndim);
 
   /* See if the total number of objects is given in the header keywords. */
   keys[0].name="NUMLABS";
@@ -669,6 +813,11 @@ ui_read_labels(struct mkcatalogparams *p)
   ui_wcs_info(p);
 
 
+  /* Make the tiles that cover each object and also correct the total
+     number of objects based on the parsing of the image. */
+  ui_one_tile_per_object_correct_numobjects(p);
+
+
   /* Read the clumps array if necessary. */
   if(p->clumpscat)
     {
@@ -741,7 +890,6 @@ ui_read_labels(struct mkcatalogparams *p)
         }
     }
 
-
   /* Clean up. */
   keys[0].name=keys[1].name=NULL;
   keys[0].array=keys[1].array=NULL;
@@ -764,6 +912,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
   size_t i;
 
   if(p->upperlimit) *values=1;
+  if(p->spectrum)   *values=*std=1;
 
   /* Go over all the object columns. Note that the objects and clumps (if
      the `--clumpcat' option is given) inputs are mandatory and it is not
@@ -773,12 +922,15 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
       switch(i)
         {
         case OCOL_NUMALL:             /* Only object labels. */    break;
+        case OCOL_NUMALLXY:           /* Only object labels. */    break;
         case OCOL_NUM:                *values        = 1;          break;
+        case OCOL_NUMXY:              *values        = 1;          break;
         case OCOL_SUM:                *values        = 1;          break;
         case OCOL_SUM_VAR:            *values = *std = 1;          break;
         case OCOL_MEDIAN:             *values        = 1;          break;
         case OCOL_VX:                 *values        = 1;          break;
         case OCOL_VY:                 *values        = 1;          break;
+        case OCOL_VZ:                 *values        = 1;          break;
         case OCOL_VXX:                *values        = 1;          break;
         case OCOL_VYY:                *values        = 1;          break;
         case OCOL_VXY:                *values        = 1;          break;
@@ -788,6 +940,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         case OCOL_NUMWHT:             *values        = 1;          break;
         case OCOL_GX:                 /* Only object labels. */    break;
         case OCOL_GY:                 /* Only object labels. */    break;
+        case OCOL_GZ:                 /* Only object labels. */    break;
         case OCOL_GXX:                /* Only object labels. */    break;
         case OCOL_GYY:                /* Only object labels. */    break;
         case OCOL_GXY:                /* Only object labels. */    break;
@@ -800,8 +953,10 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         case OCOL_C_SUM:              *values        = 1;          break;
         case OCOL_C_VX:               *values        = 1;          break;
         case OCOL_C_VY:               *values        = 1;          break;
+        case OCOL_C_VZ:               *values        = 1;          break;
         case OCOL_C_GX:               /* Only clump labels. */     break;
         case OCOL_C_GY:               /* Only clump labels. */     break;
+        case OCOL_C_GZ:               /* Only clump labels. */     break;
         case OCOL_C_SUMWHT:           *values        = 1;          break;
         case OCOL_C_NUMWHT:           *values        = 1;          break;
         default:
@@ -818,7 +973,9 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         switch(i)
           {
           case CCOL_NUMALL:           /* Only clump labels. */     break;
+          case CCOL_NUMALLXY:         /* Only clump labels. */     break;
           case CCOL_NUM:              *values        = 1;          break;
+          case CCOL_NUMXY:            *values        = 1;          break;
           case CCOL_SUM:              *values        = 1;          break;
           case CCOL_SUM_VAR:          *values = *std = 1;          break;
           case CCOL_MEDIAN:           *values        = 1;          break;
@@ -827,6 +984,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_RIV_SUM_VAR:      *values = *std = 1;          break;
           case CCOL_VX:               *values        = 1;          break;
           case CCOL_VY:               *values        = 1;          break;
+          case CCOL_VZ:               *values        = 1;          break;
           case CCOL_VXX:              *values        = 1;          break;
           case CCOL_VYY:              *values        = 1;          break;
           case CCOL_VXY:              *values        = 1;          break;
@@ -836,6 +994,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_NUMWHT:           *values        = 1;          break;
           case CCOL_GX:               /* Only clump labels. */     break;
           case CCOL_GY:               /* Only clump labels. */     break;
+          case CCOL_GZ:               /* Only clump labels. */     break;
           case CCOL_GXX:              /* Only clump labels. */     break;
           case CCOL_GYY:              /* Only clump labels. */     break;
           case CCOL_GXY:              /* Only clump labels. */     break;
@@ -843,6 +1002,8 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_MAXX:             /* Only clump labels. */     break;
           case CCOL_MINY:             /* Only clump labels. */     break;
           case CCOL_MAXY:             /* Only clump labels. */     break;
+          case CCOL_MINZ:             /* Only clump labels. */     break;
+          case CCOL_MAXZ:             /* Only clump labels. */     break;
           case CCOL_UPPERLIMIT_B:     *values        = 1;          break;
           case CCOL_UPPERLIMIT_S:     *values        = 1;          break;
           case CCOL_UPPERLIMIT_Q:     *values        = 1;          break;
@@ -998,7 +1159,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
       /* Initially, `p->hasblank' was set based on the objects image, but
          it may happen that the objects image only has zero values for
          blank pixels, so we'll also do a check on the input image. */
-      p->hasblank = gal_blank_present(p->objects, 1);
+      p->hasblank = gal_blank_present(p->values, 1);
 
       /* Reset the units of the value-based columns if the input dataset
          has defined units. */
@@ -1147,41 +1308,49 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 static void
 ui_preparations_read_keywords(struct mkcatalogparams *p)
 {
-  float minstd;
   gal_data_t *tmp;
+  float std, minstd;
   gal_data_t *keys=NULL;
 
-  /* When a Sky standard deviation dataset (not number) is given. */
-  if(p->std && p->std->size>1)
+  /* Set the counts-per-second correction. */
+  if(p->std)
     {
-      /* Read the keywords from the standard deviation image. */
-      keys=gal_data_array_calloc(2);
-      keys[0].next=&keys[1];
-      keys[0].name="MINSTD";              keys[1].name="MEDSTD";
-      keys[0].type=GAL_TYPE_FLOAT32;      keys[1].type=GAL_TYPE_FLOAT32;
-      keys[0].array=&minstd;              keys[1].array=&p->medstd;
-      gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
-
-      /* If the two keywords couldn't be read. We don't want to slow down
-         the user for the median (which needs sorting). So we'll just
-         calculate the minimum which is necessary for the `p->cpscorr'. */
-      if(keys[1].status) p->medstd=NAN;
-      if(keys[0].status)
+      if(p->std->size>1)
         {
-          /* Calculate the minimum STD. */
-          tmp=gal_statistics_minimum(p->std);
-          minstd=*((float *)(tmp->array));
-          gal_data_free(tmp);
+          /* Read the keywords from the standard deviation image. */
+          keys=gal_data_array_calloc(2);
+          keys[0].next=&keys[1];
+          keys[0].name="MINSTD";              keys[1].name="MEDSTD";
+          keys[0].type=GAL_TYPE_FLOAT32;      keys[1].type=GAL_TYPE_FLOAT32;
+          keys[0].array=&minstd;              keys[1].array=&p->medstd;
+          gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
 
-          /* If the units are in variance, then take the square root. */
-          if(p->variance) minstd=sqrt(minstd);
+          /* If the two keywords couldn't be read. We don't want to slow down
+             the user for the median (which needs sorting). So we'll just
+             calculate the minimum which is necessary for the `p->cpscorr'. */
+          if(keys[1].status) p->medstd=NAN;
+          if(keys[0].status)
+            {
+              /* Calculate the minimum STD. */
+              tmp=gal_statistics_minimum(p->std);
+              minstd=*((float *)(tmp->array));
+              gal_data_free(tmp);
+
+              /* If the units are in variance, then take the square root. */
+              if(p->variance) minstd=sqrt(minstd);
+            }
+          p->cpscorr = minstd>1 ? 1.0f : minstd;
+
+          /* Clean up. */
+          keys[0].name=keys[1].name=NULL;
+          keys[0].array=keys[1].array=NULL;
+          gal_data_array_free(keys, 2, 1);
+        }
+      else
+        {
+          std=((float *)(p->std->array))[0];
+          p->cpscorr=std>1 ? 1.0f : std;
         }
-      p->cpscorr = minstd>1 ? 1.0f : minstd;
-
-      /* Clean up. */
-      keys[0].name=keys[1].name=NULL;
-      keys[0].array=keys[1].array=NULL;
-      gal_data_array_free(keys, 2, 1);
     }
 }
 
@@ -1328,67 +1497,85 @@ ui_preparations_outnames(struct mkcatalogparams *p)
 
 
 
-/* To make the catalog processing more scalable (and later allow for
-   over-lappping regions), we will define a tile for each object. */
-void
-ui_one_tile_per_object(struct mkcatalogparams *p)
+/* When a spectrum is requested, the slice information (slice number and
+   slice WCS) is common to all different spectra. So instead of calculating
+   it every time, we'll just make it once here, then copy it for every
+   object.
+
+   The Slice information is going to be written in every spectrum. So we
+   don't want it to take too much space. Therefore, only when the number of
+   slices is less than 65535 (2^16-1), will we actually use a 32-bit
+   integer type for the slice number column.
+*/
+static void
+ui_preparations_spectrum_wcs(struct mkcatalogparams *p)
 {
-  size_t ndim=p->objects->ndim;
+  double *xarr, *yarr, *zarr;
+  gal_data_t *x, *y, *z, *coords;
+  size_t i, numslices=p->objects->dsize[0];
+  size_t slicenumtype=numslices>=65535 ? GAL_TYPE_UINT32 : GAL_TYPE_UINT16;
 
-  int32_t *l, *lf, *start;
-  size_t i, d, *min, *max, width=2*ndim;
-  size_t *minmax=gal_pointer_allocate(GAL_TYPE_SIZE_T,
-                                      width*p->numobjects, 0, __func__,
-                                      "minmax");
-  size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
-                                      "coord");
+  /* A small sanity check. */
+  if(p->objects->ndim!=3)
+    error(EXIT_FAILURE, 0, "%s (hdu %s) is a %zuD dataset, but `--spectrum' "
+          "is currently only defined on 3D datasets", p->objectsfile,
+          p->cp.hdu, p->objects->ndim);
+
+  /* Allocate space for the slice number as well as the X and Y positions
+     for WCS conversion. Note that the `z' axis is going to be converted to
+     WCS later, so we'll just give it the basic information now.*/
+  x=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
+                   p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
+  y=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
+                   p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
+  z=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
+                   p->cp.minmapsize, p->cp.quietmmap, p->ctype[2],
+                   p->objects->wcs->cunit[2], "Slice WCS coordinates.");
+
+  /* Write values into the 3 coordinates. */
+  xarr=x->array; yarr=y->array; zarr=z->array;
+  for(i=0;i<numslices;++i) { zarr[i]=i+1; xarr[i]=yarr[i]=1; }
 
 
-  /* Initialize the minimum and maximum position for each tile/object. So,
-     we'll initialize the minimum coordinates to the maximum possible
-     `size_t' value (in `GAL_BLANK_SIZE_T') and the maximums to zero. */
-  for(i=0;i<p->numobjects;++i)
-    for(d=0;d<ndim;++d)
-      {
-        minmax[ i * width +        d ] = GAL_BLANK_SIZE_T; /* Minimum. */
-        minmax[ i * width + ndim + d ] = 0;                /* Maximum. */
-      }
-
-  /* Go over the objects label image and correct the minimum and maximum
-     coordinates. */
-  start=p->objects->array;
-  lf=(l=p->objects->array)+p->objects->size;
-  do
-    if(*l>0)
-      {
-        /* Get the coordinates of this pixel. */
-        gal_dimension_index_to_coord(l-start, ndim, p->objects->dsize, coord);
-
-        /* Check to see this coordinate is the smallest/largest found so
-           far for this label. Note that labels start from 1, while indexs
-           here start from zero. */
-        min = &minmax[ (*l-1) * width        ];
-        max = &minmax[ (*l-1) * width + ndim ];
-        for(d=0;d<ndim;++d)
-          {
-            if( coord[d] < min[d] ) min[d] = coord[d];
-            if( coord[d] > max[d] ) max[d] = coord[d];
-          }
-      }
-  while(++l<lf);
+  /* Convert the coordinates to WCS. We are doing this inplace to avoid too
+     much memory/speed consumption. */
+  coords=x;
+  coords->next=y;
+  coords->next->next=z;
+  gal_wcs_img_to_world(coords, p->objects->wcs, 1);
 
   /* For a check.
-  for(i=0;i<p->numobjects;++i)
-    printf("%zu: (%zu, %zu) --> (%zu, %zu)\n", i+1, minmax[i*width],
-           minmax[i*width+1], minmax[i*width+2], minmax[i*width+3]);
+  for(i=0;i<numslices;++i)
+    printf("%g, %g, %g\n", xarr[i], yarr[i], zarr[i]);
+  exit(0);
   */
 
-  /* Make the tiles. */
-  p->tiles=gal_tile_series_from_minmax(p->objects, minmax, p->numobjects);
+  /* Allocate the slice counter array (we are doing it again because we
+     want it to be in integer type now). */
+  p->specsliceinfo=gal_data_alloc(NULL, slicenumtype, 1, &numslices, NULL, 0,
+                                  p->cp.minmapsize, p->cp.quietmmap, "SLICE",
+                                  "counter",
+                                  "Slice number in cube (counting from 1).");
+  if(p->specsliceinfo->type==GAL_TYPE_UINT16)
+    for(i=0;i<numslices;++i) ((uint16_t *)(p->specsliceinfo->array))[i]=i+1;
+  else
+    for(i=0;i<numslices;++i) ((uint32_t *)(p->specsliceinfo->array))[i]=i+1;
+
+  /* Set the slice WCS column information. Note that `z' is now the WCS
+     coordinate value of the third dimension, and to avoid wasting extra
+     space (this column is repeated one very object's spectrum), we'll
+     convert it to a 32-bit floating point dataset. */
+  p->specsliceinfo->next=gal_data_copy_to_new_type(z, GAL_TYPE_FLOAT32);
+
+  /* For a final check.
+  gal_table_write(p->specsliceinfo, NULL, GAL_TABLE_FORMAT_BFITS,
+                  "specsliceinfo.fits", "test-debug",0);
+  */
 
   /* Clean up. */
-  free(coord);
-  free(minmax);
+  gal_data_free(x);
+  gal_data_free(y);
+  gal_data_free(z);
 }
 
 
@@ -1451,9 +1638,9 @@ void
 ui_preparations(struct mkcatalogparams *p)
 {
   /* If no columns are requested, then inform the user. */
-  if(p->columnids==NULL)
-    error(EXIT_FAILURE, 0, "no columns requested, please run again with "
-          "`--help' for the full list of columns you can ask for");
+  if(p->columnids==NULL && p->spectrum==0)
+    error(EXIT_FAILURE, 0, "no measurements requested! Please run again "
+          "with `--help' for the possible list of measurements");
 
 
   /* Set the actual filenames to use. */
@@ -1481,8 +1668,12 @@ ui_preparations(struct mkcatalogparams *p)
   ui_preparations_outnames(p);
 
 
-  /* Make the tiles that cover each object. */
-  ui_one_tile_per_object(p);
+  /* If a spectrum is requested, generate the two WCS columns. */
+  if(p->spectrum)
+    {
+      ui_preparations_spectrum_wcs(p);
+      p->spectra=gal_data_array_calloc(p->numobjects);
+    }
 
 
   /* Allocate the reference random number generator and seed values. It
@@ -1667,7 +1858,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
 void
 ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
 {
-  size_t d;
+  size_t d, i;
 
   /* The temporary arrays for WCS coordinates. */
   if(p->wcs_vo ) gal_list_data_free(p->wcs_vo);
@@ -1713,9 +1904,29 @@ ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
   gal_data_free(p->upmask);
   gal_data_free(p->clumps);
   gal_data_free(p->objects);
+  if(p->outlabs) free(p->outlabs);
+  gal_list_data_free(p->clumpcols);
+  gal_list_data_free(p->objectcols);
+  gal_list_data_free(p->specsliceinfo);
   if(p->upcheckout) free(p->upcheckout);
   gal_data_array_free(p->tiles, p->numobjects, 0);
 
+  /* Clean up the spectra. */
+  if(p->spectra)
+    {
+      /* Note that each element of the array is the first node in a list of
+         datasets. So we can't free the first one with
+         `gal_list_data_free', we'll delete all the nodes after it in the
+         loop. */
+      for(i=0;i<p->numobjects;++i)
+        {
+          gal_list_data_free( p->spectra[i].next );
+          p->spectra[i].next=NULL;
+          gal_data_free_contents(&p->spectra[i]);
+        }
+      gal_data_array_free(p->spectra, p->numobjects, 0);
+    }
+
   /* If the Sky or its STD image were given in tiles, then we defined a
      tile structure to deal with them. The initialization of the tile
      structure is checked with its `ndim' element. */
