@@ -66,6 +66,7 @@ enum option_keys_enum
   UI_KEY_AREA            = 'a',
   UI_KEY_X               = 'x',
   UI_KEY_Y               = 'y',
+  UI_KEY_Z               = 'z',
   UI_KEY_RA              = 'r',
   UI_KEY_DEC             = 'd',
   UI_KEY_BRIGHTNESS      = 'b',
@@ -90,6 +91,8 @@ enum option_keys_enum
   UI_KEY_SUBTRACTSKY,
   UI_KEY_SFMAGNSIGMA,
   UI_KEY_SFMAGAREA,
+  UI_KEY_SPECTRUM,
+  UI_KEY_INBETWEENINTS,
   UI_KEY_UPMASKFILE,
   UI_KEY_UPMASKHDU,
   UI_KEY_UPNUM,
@@ -101,27 +104,38 @@ enum option_keys_enum
 
   UI_KEY_OBJID,                         /* Catalog columns. */
   UI_KEY_IDINHOSTOBJ,
+  UI_KEY_AREAXY,
   UI_KEY_CLUMPSAREA,
   UI_KEY_WEIGHTAREA,
   UI_KEY_GEOAREA,
+  UI_KEY_GEOAREAXY,
   UI_KEY_GEOX,
   UI_KEY_GEOY,
+  UI_KEY_GEOZ,
   UI_KEY_CLUMPSX,
   UI_KEY_CLUMPSY,
+  UI_KEY_CLUMPSZ,
   UI_KEY_CLUMPSGEOX,
   UI_KEY_CLUMPSGEOY,
+  UI_KEY_CLUMPSGEOZ,
   UI_KEY_MINX,
   UI_KEY_MAXX,
   UI_KEY_MINY,
   UI_KEY_MAXY,
+  UI_KEY_MINZ,
+  UI_KEY_MAXZ,
   UI_KEY_W1,
   UI_KEY_W2,
+  UI_KEY_W3,
   UI_KEY_GEOW1,
   UI_KEY_GEOW2,
+  UI_KEY_GEOW3,
   UI_KEY_CLUMPSW1,
   UI_KEY_CLUMPSW2,
+  UI_KEY_CLUMPSW3,
   UI_KEY_CLUMPSGEOW1,
   UI_KEY_CLUMPSGEOW2,
+  UI_KEY_CLUMPSGEOW3,
   UI_KEY_BRIGHTNESSERR,
   UI_KEY_CLUMPSBRIGHTNESS,
   UI_KEY_BRIGHTNESSNORIVER,
