@@ -403,7 +403,7 @@ struct argp_option program_options[] =
       UI_KEY_PCOL,
       "STR/INT",
       0,
-      "Position angle.",
+      "Position angle (First X-Z-X Euler angle in 3D).",
       UI_GROUP_CATALOG,
       &p->pcol,
       GAL_TYPE_STRING,
@@ -411,12 +411,38 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "p2col",
+      UI_KEY_P2COL,
+      "STR/INT",
+      0,
+      "Second Euler angle (X-Z-X order).",
+      UI_GROUP_CATALOG,
+      &p->p2col,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "p3col",
+      UI_KEY_P2COL,
+      "STR/INT",
+      0,
+      "Third Euler angle (X-Z-X order).",
+      UI_GROUP_CATALOG,
+      &p->p3col,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "qcol",
       UI_KEY_QCOL,
       "STR/INT",
       0,
-      "Axis ratio.",
+      "Axis ratio (major/dim2 in 3D).",
       UI_GROUP_CATALOG,
       &p->qcol,
       GAL_TYPE_STRING,
@@ -424,6 +450,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "q2col",
+      UI_KEY_Q2COL,
+      "STR/INT",
+      0,
+      "Axis ratio (major/dim3 in 3D).",
+      UI_GROUP_CATALOG,
+      &p->q2col,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "mcol",
       UI_KEY_MCOL,
