@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -96,7 +96,7 @@ detection_initial(struct noisechiselparams *p)
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "Eroded %zu time%s (%zu-connected).", p->erode,
-                   p->erode>1?"s":"", p->erodengb)<0 )
+                   p->erode!=1?"s":"", p->erodengb)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
@@ -247,7 +247,7 @@ detection_fill_holes_open(void *in_prm)
   /* A temporary data structure to wrap around the copy space. Note that
      the initially allocated space for this tile is only 1 pixel! */
   copy=gal_data_alloc(NULL, GAL_TYPE_UINT8, p->input->ndim, dsize,
-                      NULL, 0, -1, NULL, NULL, NULL);
+                      NULL, 0, -1, 1, NULL, NULL, NULL);
   free(copy->array);
   copy->array=&fho_prm->copyspace[p->maxltcontig*tprm->id];
 
@@ -293,7 +293,7 @@ detection_fill_holes_open(void *in_prm)
         }
 
       /* Open all the regions. */
-      gal_binary_open(copy, 1, 1, 1);
+      gal_binary_open(copy, p->dopening, p->dopeningngb==4 ? 1 : 2, 1);
 
       /* Write the copied region back into the large input and AFTERWARDS,
          correct the tile's pointers, the pointers must not be corrected
@@ -327,6 +327,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
   float *f;
   uint8_t *b, *bf;
   gal_data_t *bin;
+  int con=p->pseudoconcomp==4 ? 1 : 2;
   struct fho_params fho_prm={0, NULL, workbin, worklab, p};
 
 
@@ -441,7 +442,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
       do if(*b==GAL_BLANK_UINT8) *b = !s0d1; while(++b<bf);
     }
   */
-  return gal_binary_connected_components(workbin, &worklab, 1);
+  return gal_binary_connected_components(workbin, &worklab, con);
 }
 
 
@@ -483,13 +484,6 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
                 : "SKY_PSEUDODET_SN" );
   threshold_write_sn_table(p, sn, snind, str, comments, extname);
   gal_list_str_free(comments, 1);
-
-
-  /* Abort NoiseChisel if the user asked for it. */
-  if(s0d1D2==2 && !p->continueaftercheck)
-    ui_abort_after_check(p, p->detsn_s_name, p->detsn_d_name,
-                         "pseudo-detection and grown/final detection S/N "
-                         "values in a table");
 }
 
 
@@ -541,12 +535,12 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
                                          "flag")
                  : NULL );
   sn         = gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &tablen, NULL, 1,
-                              p->cp.minmapsize, "SIGNAL-TO-NOISE", "ratio",
-                              NULL);
+                              p->cp.minmapsize, p->cp.quietmmap,
+                              "SIGNAL-TO-NOISE", "ratio", NULL);
   snind      = ( p->checksn==0 ? NULL
                  : gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &tablen, NULL, 1,
-                                  p->cp.minmapsize, "LABEL", "counter",
-                                  NULL) );
+                                  p->cp.minmapsize, p->cp.quietmmap, "LABEL",
+                                  "counter", NULL) );
 
   /* Go over all the pixels and get the necessary information. */
   fs = f = p->input->array;
@@ -757,40 +751,48 @@ detection_pseudo_real(struct noisechiselparams *p)
   worklab=gal_data_copy(p->olabel);
   workbin=gal_data_alloc(NULL, GAL_TYPE_UINT8, p->input->ndim,
                          p->input->dsize, p->input->wcs, 0, p->cp.minmapsize,
-                         NULL, NULL, NULL);
+                         p->cp.quietmmap, NULL, NULL, NULL);
   workbin->flag=p->input->flag;
 
 
-  /* Over the Sky: find the pseudo-detections and make the S/N table. */
-  if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  numpseudo=detection_pseudo_find(p, workbin, worklab, 0);
-  sn=detection_sn(p, worklab, numpseudo, 0, "PSEUDOS-FOR-SN");
-
-
-  /* A small sanity check */
-  if( sn->size < p->minnumfalse)
-    error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found over "
-          "the sky region to estimate an S/N. This is less than %zu (value "
-          "to `--minnumfalse' option). Please adjust parameters like "
-          "`--dthresh', `--snminarea', or make sure that there actually "
-          "is sufficient sky area after initial detection. You can use "
-          "`--checkdetection' to see every step until this point", sn->size,
-          p->minnumfalse);
-
-
-  /* Get the S/N quantile and report it if we are in non-quiet mode. */
-  quant=gal_statistics_quantile(sn, p->snquant, 1);
-  p->detsnthresh = *((float *)(quant->array));
-  if(!p->cp.quiet)
+  /* If a manual S/N is given, then don't bother with using
+     pseudo-detections. */
+  if( isnan(p->snthresh) )
     {
-      if( asprintf(&msg, "Pseudo-det S/N: %.3f (%g quant of %zu).",
-                   p->detsnthresh, p->snquant, sn->size)<0 )
-        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-      gal_timing_report(&t1, msg, 2);
-      free(msg);
+      /* Over the Sky: find the pseudo-detections and make the S/N
+         table. */
+      if(!p->cp.quiet) gettimeofday(&t1, NULL);
+      numpseudo=detection_pseudo_find(p, workbin, worklab, 0);
+      sn=detection_sn(p, worklab, numpseudo, 0, "PSEUDOS-FOR-SN");
+
+
+      /* A small sanity check */
+      if( sn->size < p->minnumfalse)
+        error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found "
+              "over the sky region to estimate an S/N. This is less than "
+              "%zu (value to `--minnumfalse' option). Please adjust "
+              "parameters like `--dthresh', `--snminarea', or make sure "
+              "that there actually is sufficient sky area after initial "
+              "detection. You can use `--checkdetection' to see every step "
+              "until this point", sn->size, p->minnumfalse);
+
+
+      /* Get the S/N quantile and report it if we are in non-quiet mode. */
+      quant=gal_statistics_quantile(sn, p->snquant, 1);
+      p->detsnthresh = *((float *)(quant->array));
+      if(!p->cp.quiet)
+        {
+          if( asprintf(&msg, "Pseudo-det S/N: %.3f (%g quant of %zu).",
+                       p->detsnthresh, p->snquant, sn->size)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+          gal_timing_report(&t1, msg, 2);
+          free(msg);
+        }
+      gal_data_free(sn);
+      gal_data_free(quant);
     }
-  gal_data_free(sn);
-  gal_data_free(quant);
+  else
+    p->detsnthresh=p->snthresh;
 
 
   /* Over the detections: find pseudo-detections and make S/N table. */
@@ -821,23 +823,19 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
   size_t i;
   int8_t *b;
   float *snarr;
-  gal_data_t *sn, *snind;
+  gal_data_t *sn;
   int32_t *l, *lf, curlab=1;
-  gal_list_str_t *comments=NULL;
-  char *extname="GROWN_DETECTION_SN";
   int32_t *newlabs=gal_pointer_allocate(GAL_TYPE_INT32, num+1, 1, __func__,
                                         "newlabs");
 
   /* Get the Signal to noise ratio of all detections. */
   sn=detection_sn(p, p->olabel, num, 2, "DILATED");
 
-
   /* Only keep the objects with an S/N above the pseudo-detection limit. */
   snarr=sn->array;
   for(i=1;i<num+1;++i)
     newlabs[i] = snarr[i] > p->detsnthresh ? curlab++ : 0;
 
-
   /* Go over the labeled image and correct the labels. */
   b=workbin->array;
   lf=(l=p->olabel->array)+p->olabel->size;
@@ -853,32 +851,6 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
     }
   else do *b++ = (*l=newlabs[ *l ]) > 0; while(++l<lf);
 
-
-  /* Save the S/N values if the user asked for them. */
-  if(p->detsn_D_name)
-    {
-      /* Make the S/N index array. */
-      snind=gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &num, NULL, 0,
-                           p->cp.minmapsize, NULL, NULL, NULL);
-
-      /* Fill in the indexs. Note that the S/N array had num+1 elements, so
-         we also want to shift them back by one element, so we also need to
-         correct its size. */
-      l=snind->array;
-      sn->size = sn->dsize[0] = num;
-      for(i=0;i<num;++i) { l[i]=i+1; snarr[i]=snarr[i+1]; }
-
-      /* Make the comments, then write the table. */
-      gal_list_str_add(&comments, "See also: `DILATED' "
-                       "HDU of output with `--checkdetection'.", 1);
-      gal_list_str_add(&comments, "S/N of finally grown detections.", 1);
-      threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments,
-                               extname);
-      gal_list_str_free(comments, 1);
-
-    }
-
-
   /* Clean up and return. */
   free(newlabs);
   gal_data_free(sn);
@@ -982,9 +954,11 @@ detection_remove_false_initial(struct noisechiselparams *p,
       e_th=p->exp_thresh_full->array;
       do                                    /* Growth is necessary later.  */
         {                                   /* So there is no need to set  */
-          if(*l!=GAL_BLANK_INT32)           /* the labels image, but we    */
-            {                               /* have to count the number of */
-              *b = newlabels[ *l ] > 0;     /* pixels to (possibly) grow.  */
+          if(*l==GAL_BLANK_INT32)           /* the labels image, but we    */
+            *b=GAL_BLANK_UINT8;             /* have to count the number of */
+          else                              /* pixels to (possibly) grow.  */
+            {
+              *b = newlabels[ *l ] > 0;
               if( *b==0 && *arr>*e_th )
                 ++p->numexpand;
             }
@@ -992,11 +966,11 @@ detection_remove_false_initial(struct noisechiselparams *p,
         }
       while(++l<lf);
 
+
       /* If there aren't any pixels to later expand, then reset the labels
          (remove false detections in the labeled image). */
       if(p->numexpand==0)
         {
-          b=workbin->array;
           l=p->olabel->array;
           do if(*l!=GAL_BLANK_INT32) *l = newlabels[ *l ]; while(++l<lf);
         }
@@ -1031,8 +1005,8 @@ detection_quantile_expand(struct noisechiselparams *p, gal_data_t *workbin)
       /* Allocate the space necessary to keep the index of all the pixels
          that must be expanded and re-initialize the necessary pointers. */
       diffuseindexs=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &p->numexpand,
-                                   NULL, 0, p->cp.minmapsize, NULL, NULL,
-                                   NULL);
+                                   NULL, 0, p->cp.minmapsize, p->cp.quietmmap,
+                                   NULL, NULL, NULL);
 
       /* Fill in the diffuse indexs and initialize the objects dataset. */
       b    = workbin->array;
