@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -40,13 +40,18 @@ threshold_apply(struct noisechiselparams *p, float *value1, float *value2,
 void
 threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *sntable,
                          gal_data_t *snind, char *filename,
-                         gal_list_str_t *comments);
+                         gal_list_str_t *comments, char *extname);
 
 void
 threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
                         gal_data_t **second, gal_data_t **third,
                         char *filename);
 
+void
+threshold_no_outlier(struct noisechiselparams *p, gal_data_t *first,
+                     gal_data_t *second, gal_data_t *third,
+                     char *filename);
+
 void
 threshold_quantile_find_apply(struct noisechiselparams *p);
 
