@@ -107,6 +107,7 @@ ui_initialize_options(struct noisechiselparams *p,
 
 
   /* Set the necessary common parameters structure. */
+  cp->program_struct     = p;
   cp->poptions           = program_options;
   cp->program_name       = PROGRAM_NAME;
   cp->program_exec       = PROGRAM_EXEC;
@@ -230,24 +231,6 @@ ui_read_check_only_options(struct noisechiselparams *p)
           "and avoid convolution) it is mandatory to also specify a HDU "
           "for it");
 
-  /* Make sure the connectivities have the correct values. */
-  if(p->erodengb!=4 && p->erodengb!=8)
-    error(EXIT_FAILURE, 0, "%zu not acceptable for `--erodengb'. It must "
-          "be 4 or 8 (specifying the type of connectivity)", p->erodengb);
-  if(p->openingngb!=4 && p->openingngb!=8)
-    error(EXIT_FAILURE, 0, "%zu not acceptable for `--openingngb'. It must "
-          "be 4 or 8 (specifying the type of connectivity)", p->openingngb);
-  if(p->dopeningngb!=4 && p->dopeningngb!=8)
-    error(EXIT_FAILURE, 0, "%zu not acceptable for `--dopeningngb'. It must "
-          "be 4 or 8 (specifying the type of connectivity)", p->dopeningngb);
-  if(p->holengb!=4 && p->holengb!=8)
-    error(EXIT_FAILURE, 0, "%zu not acceptable for `--holengb'. It must "
-          "be 4 or 8 (specifying the type of connectivity)", p->holengb);
-  if(p->pseudoconcomp!=4 && p->pseudoconcomp!=8)
-    error(EXIT_FAILURE, 0, "%zu not acceptable for `--pseudoconcomp'. It "
-          "must be 4 or 8 (specifying the type of connectivity)",
-          p->pseudoconcomp);
-
   /* Make sure that the no-erode-quantile is not smaller or equal to
      qthresh. */
   if( p->noerodequant <= p->qthresh)
@@ -437,7 +420,7 @@ ui_prepare_kernel(struct noisechiselparams *p)
     {
       if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
         p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize, p->cp.quietmmap);
       else
         p->kernel=NULL;
     }
@@ -447,7 +430,7 @@ ui_prepare_kernel(struct noisechiselparams *p)
          allocated array. */
       p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 2,
                                kernel_2d_dsize, NULL, 0, p->cp.minmapsize,
-                               NULL, NULL, NULL);
+                               p->cp.quietmmap, NULL, NULL, NULL);
 
       /* Now copy the staticly allocated array into it. */
       k=p->kernel->array;
@@ -460,7 +443,7 @@ ui_prepare_kernel(struct noisechiselparams *p)
      ignore it. */
   if(p->widekernelname)
     p->widekernel=gal_fits_img_read_kernel(p->widekernelname, p->whdu,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize, p->cp.quietmmap);
 }
 
 
@@ -546,28 +529,68 @@ ui_prepare_tiles(struct noisechiselparams *p)
 
 
 
+static void
+ui_ngb_check(size_t value, char *optionname, size_t ndim)
+{
+  switch(ndim)
+    {
+    case 2:
+      if(value!=4 && value!=8)
+        error(EXIT_FAILURE, 0, "%zu is not an acceptable value for "
+              "`--%s'. Acceptable values for 2D inputs are 4 or 8",
+              value, optionname);
+      break;
+    case 3:
+      error(EXIT_FAILURE, 0, "3D input data is not yet supported");
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+            "problem. Dimension value %zu is not recognized.", __func__,
+            PACKAGE_BUGREPORT, ndim);
+    }
+}
+
+
+
+
+
 /* Read the input image and do the basic checks */
 static void
 ui_preparations_read_input(struct noisechiselparams *p)
 {
   float *f;
+  size_t ndim;
 
-  /* Read the input as a single precision floating point dataset. */
+  /* Read the input as a single precision floating point dataset, also load
+     the WCS and finally remove any possibly existing extra dimensions
+     (with a length of 1). */
   p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize,
+                                           p->cp.quietmmap);
   p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
                                &p->input->nwcs);
+  p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                            p->input->dsize,
+                                            p->input->wcs);
+
+  /* When the input doesn't have a name, use `INPUT'. */
   if(p->input->name==NULL)
     gal_checkset_allocate_copy("INPUT", &p->input->name);
 
-
   /* NoiseChisel currently only works on 2D datasets (images). */
   if(p->input->ndim!=2)
     error(EXIT_FAILURE, 0, "%s (hdu: %s) has %zu dimensions but NoiseChisel "
           "can only operate on 2D datasets (images)", p->inputname, p->cp.hdu,
           p->input->ndim);
 
+  /* Check the values of dimension-related options. */
+  ndim=p->input->ndim;
+  ui_ngb_check(p->holengb, "holengb", ndim);
+  ui_ngb_check(p->erodengb, "erodengb", ndim);
+  ui_ngb_check(p->openingngb, "openingngb", ndim);
+  ui_ngb_check(p->dopeningngb, "dopeningngb", ndim);
+  ui_ngb_check(p->pseudoconcomp, "pseudoconcomp", ndim);
+
   /* A small check to see if the edges of the dataset aren't zero valued:
      they should be masked. */
   f=p->input->array;
@@ -613,7 +636,8 @@ ui_preparations(struct noisechiselparams *p)
       /* Read the input convolved image. */
       p->conv = gal_array_read_one_ch_to_type(p->convolvedname, p->chdu,
                                               NULL, GAL_TYPE_FLOAT32,
-                                              p->cp.minmapsize);
+                                              p->cp.minmapsize,
+                                              p->cp.quietmmap);
 
       /* Make sure the convolved image is the same size as the input. */
       if( gal_dimension_is_different(p->input, p->conv) )
@@ -634,10 +658,12 @@ ui_preparations(struct noisechiselparams *p)
   /* Allocate space for the over-all necessary arrays. */
   p->binary=gal_data_alloc(NULL, GAL_TYPE_UINT8, p->input->ndim,
                            p->input->dsize, p->input->wcs, 0,
-                           p->cp.minmapsize, NULL, "binary", NULL);
+                           p->cp.minmapsize, p->cp.quietmmap, NULL,
+                           "binary", NULL);
   p->olabel=gal_data_alloc(NULL, GAL_TYPE_INT32, p->input->ndim,
                            p->input->dsize, p->input->wcs, 0,
-                           p->cp.minmapsize, NULL, "labels", NULL);
+                           p->cp.minmapsize, p->cp.quietmmap, NULL,
+                           "labels", NULL);
   p->binary->flag = p->olabel->flag = p->input->flag;
 }
 
