@@ -46,6 +46,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui.h"
 #include "sky.h"
+#include "contour.h"
 #include "statistics.h"
 
 
@@ -1024,6 +1025,12 @@ statistics(struct statisticsparams *p)
       print_basic_info=0;
     }
 
+  if(p->contour)
+    {
+      contour(p);
+      print_basic_info=0;
+    }
+
   /* Print the ASCII plots if requested. */
   if(p->asciihist || p->asciicfp)
     {
