@@ -509,18 +509,18 @@ _GL_WARN_ON_USE (pthread_attr_init, "pthread_attr_init is not portable - "
 #   define pthread_attr_getdetachstate rpl_pthread_attr_getdetachstate
 #  endif
 _GL_FUNCDECL_RPL (pthread_attr_getdetachstate, int,
-                  (pthread_attr_t *attr, int *detachstatep)
+                  (const pthread_attr_t *attr, int *detachstatep)
                   _GL_ARG_NONNULL ((1, 2)));
 _GL_CXXALIAS_RPL (pthread_attr_getdetachstate, int,
-                  (pthread_attr_t *attr, int *detachstatep));
+                  (const pthread_attr_t *attr, int *detachstatep));
 # else
 #  if !@HAVE_PTHREAD_ATTR_GETDETACHSTATE@
 _GL_FUNCDECL_SYS (pthread_attr_getdetachstate, int,
-                  (pthread_attr_t *attr, int *detachstatep)
+                  (const pthread_attr_t *attr, int *detachstatep)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
 _GL_CXXALIAS_SYS (pthread_attr_getdetachstate, int,
-                  (pthread_attr_t *attr, int *detachstatep));
+                  (const pthread_attr_t *attr, int *detachstatep));
 # endif
 _GL_CXXALIASWARN (pthread_attr_getdetachstate);
 #elif defined GNULIB_POSIXCHECK
@@ -804,8 +804,10 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_gettype, int,
                   (const pthread_mutexattr_t *attr, int *typep)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
-_GL_CXXALIAS_SYS (pthread_mutexattr_gettype, int,
-                  (const pthread_mutexattr_t *attr, int *typep));
+/* Need to cast, because on FreeBSD the first parameter is
+                        pthread_mutexattr_t *attr.  */
+_GL_CXXALIAS_SYS_CAST (pthread_mutexattr_gettype, int,
+                       (const pthread_mutexattr_t *attr, int *typep));
 # endif
 _GL_CXXALIASWARN (pthread_mutexattr_gettype);
 #elif defined GNULIB_POSIXCHECK
@@ -860,10 +862,14 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_getrobust, int,
                   (const pthread_mutexattr_t *attr, int *robustp)
                   _GL_ARG_NONNULL ((1, 2)));
 #  endif
-_GL_CXXALIAS_SYS (pthread_mutexattr_getrobust, int,
-                  (const pthread_mutexattr_t *attr, int *robustp));
+/* Need to cast, because on FreeBSD the first parameter is
+                        pthread_mutexattr_t *attr.  */
+_GL_CXXALIAS_SYS_CAST (pthread_mutexattr_getrobust, int,
+                       (const pthread_mutexattr_t *attr, int *robustp));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_getrobust);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_getrobust
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_GETROBUST
@@ -892,7 +898,9 @@ _GL_FUNCDECL_SYS (pthread_mutexattr_setrobust, int,
 _GL_CXXALIAS_SYS (pthread_mutexattr_setrobust, int,
                   (pthread_mutexattr_t *attr, int robust));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutexattr_setrobust);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutexattr_setrobust
 # if HAVE_RAW_DECL_PTHREAD_MUTEXATTR_SETROBUST
@@ -942,7 +950,9 @@ _GL_FUNCDECL_SYS (pthread_mutex_lock, int, (pthread_mutex_t *mutex)
 #  endif
 _GL_CXXALIAS_SYS (pthread_mutex_lock, int, (pthread_mutex_t *mutex));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_mutex_lock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_mutex_lock
 # if HAVE_RAW_DECL_PTHREAD_MUTEX_LOCK
@@ -1690,7 +1700,9 @@ _GL_FUNCDECL_SYS (pthread_spin_init, int,
 _GL_CXXALIAS_SYS (pthread_spin_init, int,
                   (pthread_spinlock_t *lock, int shared_across_processes));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_spin_init);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_spin_init
 # if HAVE_RAW_DECL_PTHREAD_SPIN_INIT
@@ -1715,7 +1727,9 @@ _GL_FUNCDECL_SYS (pthread_spin_lock, int, (pthread_spinlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_spin_lock, int, (pthread_spinlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_spin_lock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_spin_lock
 # if HAVE_RAW_DECL_PTHREAD_SPIN_LOCK
@@ -1740,7 +1754,9 @@ _GL_FUNCDECL_SYS (pthread_spin_trylock, int, (pthread_spinlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_spin_trylock, int, (pthread_spinlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_spin_trylock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_spin_trylock
 # if HAVE_RAW_DECL_PTHREAD_SPIN_TRYLOCK
@@ -1765,7 +1781,9 @@ _GL_FUNCDECL_SYS (pthread_spin_unlock, int, (pthread_spinlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_spin_unlock, int, (pthread_spinlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_spin_unlock);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_spin_unlock
 # if HAVE_RAW_DECL_PTHREAD_SPIN_UNLOCK
@@ -1790,7 +1808,9 @@ _GL_FUNCDECL_SYS (pthread_spin_destroy, int, (pthread_spinlock_t *lock)
 #  endif
 _GL_CXXALIAS_SYS (pthread_spin_destroy, int, (pthread_spinlock_t *lock));
 # endif
+# if __GLIBC__ >= 2
 _GL_CXXALIASWARN (pthread_spin_destroy);
+# endif
 #elif defined GNULIB_POSIXCHECK
 # undef pthread_spin_destroy
 # if HAVE_RAW_DECL_PTHREAD_SPIN_DESTROY
