@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -81,6 +81,22 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 #define GAL_BLANK_SIZE_T     GAL_BLANK_UINT64
 #endif
 
+#if GAL_CONFIG_SIZEOF_LONG == 4
+#define GAL_BLANK_LONG  GAL_BLANK_INT32
+#define GAL_BLANK_ULONG GAL_BLANK_UINT32
+#elif GAL_CONFIG_SIZEOF_LONG == 8
+#define GAL_BLANK_LONG  GAL_BLANK_INT64
+#define GAL_BLANK_ULONG GAL_BLANK_UINT64
+#endif
+
+#if GAL_CONFIG_SIZEOF_INT == 4
+#define GAL_BLANK_INT  GAL_BLANK_INT32
+#define GAL_BLANK_UINT GAL_BLANK_UINT32
+#elif GAL_CONFIG_SIZEOF_INT == 2
+#define GAL_BLANK_INT  GAL_BLANK_INT16
+#define GAL_BLANK_UINT GAL_BLANK_UINT16
+#endif
+
 
 /* Functions. */
 void
@@ -92,6 +108,9 @@ gal_blank_alloc_write(uint8_t type);
 void
 gal_blank_initialize(gal_data_t *input);
 
+void
+gal_blank_initialize_array(void *array, size_t size, uint8_t type);
+
 char *
 gal_blank_as_string(uint8_t type, int width);
 
@@ -101,13 +120,20 @@ gal_blank_is(void *pointer, uint8_t type);
 int
 gal_blank_present(gal_data_t *input, int updateflag);
 
+size_t
+gal_blank_number(gal_data_t *input, int updateflag);
+
 gal_data_t *
 gal_blank_flag(gal_data_t *data);
 
+void
+gal_blank_flag_apply(gal_data_t *input, gal_data_t *flag);
+
 void
 gal_blank_remove(gal_data_t *data);
 
-
+void
+gal_blank_remove_realloc(gal_data_t *input);
 
 
 __END_C_DECLS    /* From C++ preparations */
