@@ -115,6 +115,14 @@ gal_dimension_collapse_minmax(gal_data_t *in, size_t c_dim, int max1_min0);
 
 
 
+/************************************************************************/
+/********************             Other            **********************/
+/************************************************************************/
+size_t
+gal_dimension_remove_extra(size_t ndim, size_t *dsize, struct wcsprm *wcs);
+
+
+
 /************************************************************************/
 /********************          Neighbors           **********************/
 /************************************************************************/
