@@ -3,9 +3,9 @@ Functions to facilitate using threads.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,17 +26,17 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Include other headers if necessary here. Note that other header files
    must be included before the C++ preparations below */
 #include <pthread.h>
+#include <gnuastro/blank.h>
 
-/* When this header is included within Gnuastro's building process,
-   `IN_GNUASTRO_BUILD' is defined. In the build process, installation
-   information (in particular `GAL_GNUASTRO_HAVE_PTHREAD_BARRIER' that we
-   need below) is kept in `config.h'. When building a user's programs, this
-   information is kept in `gnuastro/gnuastro.h'. Note that all `.c' files
-   must start with the inclusion of `config.h' and that
-   `gnuastro/gnuastro.h' is only created at installation time (not present
-   during the building of Gnuastro).*/
+/* When we are within Gnuastro's building process, `IN_GNUASTRO_BUILD' is
+   defined. In the build process, installation information (in particular
+   `GAL_CONFIG_HAVE_PTHREAD_BARRIER' that we need below) is kept in
+   `config.h'. When building a user's programs, this information is kept in
+   `gnuastro/config.h'. Note that all `.c' files must start with the
+   inclusion of `config.h' and that `gnuastro/config.h' is only created at
+   installation time (not present during the building of Gnuastro).*/
 #ifndef IN_GNUASTRO_BUILD
-#include <gnuastro/gnuastro.h>
+#include <gnuastro/config.h>
 #endif
 
 /* C++ Preparations */
@@ -58,15 +58,12 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 
 
 
-/* Constant to use for non-existant index */
-#define GAL_THREADS_NON_THRD_INDEX (size_t)(-1)
-
 
 
 /*****************************************************************/
 /*********    Implementation of pthread_barrier    ***************/
 /*****************************************************************/
-#if GAL_GNUASTRO_HAVE_PTHREAD_BARRIER == 0
+#if GAL_CONFIG_HAVE_PTHREAD_BARRIER == 0
 
 /* Integer number of nano-seconds that `pthread_barrier_destroy' should
    wait for a check to see if all barriers have been reached. */
@@ -93,13 +90,18 @@ pthread_barrier_wait(pthread_barrier_t *b);
 int
 pthread_barrier_destroy(pthread_barrier_t *b);
 
-#endif  /* GAL_GNUASTRO_HAVE_PTHREAD_BARRIER == 0 */
+#endif  /* GAL_CONFIG_HAVE_PTHREAD_BARRIER == 0 */
 
 
 
-/*****************************************************************/
-/****************      gnuastro functions       ******************/
-/*****************************************************************/
+
+
+/*******************************************************************/
+/************              Thread utilities           **************/
+/*******************************************************************/
+size_t
+gal_threads_number();
+
 void
 gal_threads_dist_in_threads(size_t numactions, size_t numthreads,
                             size_t **outthrds, size_t *outthrdcols);
@@ -110,6 +112,23 @@ gal_threads_attr_barrier_init(pthread_attr_t *attr, pthread_barrier_t *b,
 
 
 
+
+/*******************************************************************/
+/************     Run a function on multiple threads  **************/
+/*******************************************************************/
+struct gal_threads_params
+{
+  size_t            id; /* Id of this thread.                            */
+  void         *params; /* Input structure for higher-level settings.    */
+  size_t       *indexs; /* Indexes of actions to be done in this thread. */
+  pthread_barrier_t *b; /* Pointer the barrier for all threads.          */
+};
+
+void
+gal_threads_spin_off(void *(*worker)(void *), void *caller_params,
+                     size_t numactions, size_t numthreads);
+
+
 __END_C_DECLS    /* From C++ preparations */
 
 #endif           /* __GAL_THREADS_H__ */
