@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -113,6 +113,14 @@ enum gal_types
 #define GAL_TYPE_ULONG GAL_TYPE_UINT64
 #endif
 
+#if GAL_CONFIG_SIZEOF_INT == 2
+#define GAL_TYPE_INT  GAL_TYPE_INT16
+#define GAL_TYPE_UINT GAL_TYPE_UINT16
+#elif GAL_CONFIG_SIZEOF_INT == 4
+#define GAL_TYPE_INT  GAL_TYPE_INT32
+#define GAL_TYPE_UINT GAL_TYPE_UINT32
+#endif
+
 
 
 
@@ -137,6 +145,9 @@ gal_type_min(uint8_t type, void *in);
 void
 gal_type_max(uint8_t type, void *in);
 
+int
+gal_type_is_int(uint8_t type);
+
 int
 gal_type_is_list(uint8_t type);
 
