@@ -4,7 +4,7 @@ Function to parse options and configuration file values.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -38,6 +38,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/threads.h>
 #include <gnuastro/pointer.h>
 #include <gnuastro/arithmetic.h>
+#include <gnuastro/interpolate.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/options.h>
@@ -250,26 +251,26 @@ gal_options_print_citation(struct argp_option *option, char *arg,
   struct gal_options_common_params *cp=(struct gal_options_common_params *)pa;
   char *gnuastro_acknowledgement;
   char *gnuastro_bibtex=
-    "Gnuastro package/infrastructure\n"
-    "-------------------------------\n"
-    "  @ARTICLE{2015ApJS..220....1A,\n"
+    "First paper introducing Gnuastro\n"
+    "--------------------------------\n"
+    "  @ARTICLE{gnuastro,\n"
     "     author = {{Akhlaghi}, M. and {Ichikawa}, T.},\n"
     "      title = \"{Noise-based Detection and Segmentation of Nebulous "
     "Objects}\",\n"
-    "    journal = {\\apjs},\n"
+    "    journal = {ApJS},\n"
     "  archivePrefix = \"arXiv\",\n"
     "     eprint = {1505.01664},\n"
     "   primaryClass = \"astro-ph.IM\",\n"
-    "   keywords = {galaxies: irregular, galaxies: photometry, "
-    "galaxies: structure, methods: data analysis, "
-    "techniques: image processing, techniques: photometric},\n"
+    "   keywords = {galaxies: irregular, galaxies: photometry,\n"
+    "               galaxies: structure, methods: data analysis,\n"
+    "               techniques: image processing, techniques: photometric},\n"
     "       year = 2015,\n"
     "      month = sep,\n"
     "     volume = 220,\n"
     "        eid = {1},\n"
     "      pages = {1},\n"
     "        doi = {10.1088/0067-0049/220/1/1},\n"
-    "     adsurl = {http://adsabs.harvard.edu/abs/2015ApJS..220....1A},\n"
+    "     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},\n"
     "    adsnote = {Provided by the SAO/NASA Astrophysics Data System}\n"
     "  }";
 
@@ -339,6 +340,62 @@ gal_options_print_citation(struct argp_option *option, char *arg,
 
 
 
+void *
+gal_options_check_config(struct argp_option *option, char *arg,
+                          char *filename, size_t lineno, void *junk)
+{
+  char *str;
+
+  /* First see if we are reading or writing. */
+  if(lineno==-1)
+    {
+      gal_checkset_allocate_copy("1", &str);
+      return str;
+    }
+
+  /* Check if the given value is different from this version. */
+  else
+    {
+      /* If its already set then ignore it. */
+      if(option->set) return NULL;
+
+      /* Activate the option and let the user know its activated. */
+      (*(uint8_t *)(option->value)) = 1;
+      printf("-----------------\n"
+             "Parsing of options AFTER `--checkconfig'.\n\n"
+             "IMPORTANT: Any option that was parsed before encountering "
+             "`--checkconfig', on the command-line or in a configuration "
+             "file, is not shown here. It is thus recommended to use this "
+             "option before calling any other option.\n"
+             "-----------------\n");
+
+      /* Print where this option was first seen: if `checkconfig' is called
+         within a configuration file, `filename!=NULL' and has an argument
+         (=="1"). But on the command-line, it has no argument or
+         filename. */
+      if(filename)
+        printf("%s:\n", filename);
+      else
+        {
+          if(arg)
+            error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                  "fix the problem. `filename==NULL', but `arg!=NULL'",
+                  __func__, PACKAGE_BUGREPORT);
+          else
+            printf("Command-line:\n");
+        }
+
+      /* Just to avoid compiler warnings for unused variables. The program
+         will never reach this point! */
+      arg=filename=NULL; lineno=0; option=NULL; junk=NULL;
+    }
+  return NULL;
+}
+
+
+
+
+
 void *
 gal_options_read_type(struct argp_option *option, char *arg,
                       char *filename, size_t lineno, void *junk)
@@ -455,6 +512,55 @@ gal_options_read_tableformat(struct argp_option *option, char *arg,
 
 
 
+void *
+gal_options_read_interpmetric(struct argp_option *option, char *arg,
+                              char *filename, size_t lineno, void *junk)
+{
+  char *str=NULL;
+  if(lineno==-1)
+    {
+      switch(*(uint8_t *)(option->value))
+        {
+        case GAL_INTERPOLATE_CLOSE_METRIC_RADIAL:
+          gal_checkset_allocate_copy("radial", &str);
+          break;
+        case GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN:
+          gal_checkset_allocate_copy("manhattan", &str);
+          break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                "fix the problem. The code %u is not recognized as a "
+                "nearest-neighbor interpolation metric", __func__,
+                PACKAGE_BUGREPORT, *(uint8_t *)(option->value));
+        }
+      return str;
+    }
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Set the value. */
+      if(       !strcmp(arg, "radial") )
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_RADIAL;
+      else if ( !strcmp(arg, "manhattan") )
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN;
+      else
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "`%s' (value to "
+                      "`%s' option) isn't valid. Currently only `radial' "
+                      "and `manhattan' metrics are recognized for nearest "
+                      "neighbor interpolation", arg, option->name);
+
+      /* For no un-used variable warning. This function doesn't need the
+         pointer. */
+      return junk=NULL;
+    }
+}
+
+
+
+
+
 /* The input to this function is a string of any number of numbers
    separated by a comma (`,') and possibly containing fractions, for
    example: `1,2/3, 4.95'. The output `gal_data_t' contains the array of
@@ -469,12 +575,15 @@ gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
   gal_list_f64_t *list=NULL, *tdll;
   double numerator=NAN, denominator=NAN, tmp;
 
-
   /* The nature of the arrays/numbers read here is very small, so since
      `p->cp.minmapsize' might not have been read yet, we will set it to -1
      (largest size_t number), so the values are kept in memory. */
+  int quietmmap=1;
   size_t minmapsize=-1;
 
+  /* If we have an empty string, just return NULL. */
+  if(string==NULL || *string=='\0') return NULL;
+
   /* Go through the input character by character. */
   while(string && *c!='\0')
     {
@@ -489,6 +598,7 @@ gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
 
         /* Comma marks the transition to the next number. */
         case ',':
+        case ':':
           if(isnan(numerator))
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "a number "
                           "must be given before `,'. You have given: `%s'",
@@ -557,7 +667,7 @@ gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
     {
       i=num;
       out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &num, NULL, 0,
-                         minmapsize, NULL, NULL, NULL);
+                         minmapsize, quietmmap, NULL, NULL, NULL);
       for(tdll=list;tdll!=NULL;tdll=tdll->next)
         ((double *)(out->array))[--i]=tdll->v;
     }
@@ -568,7 +678,7 @@ gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
          we'll allocate space for one element, then free it. */
       i=1;
       out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &i, NULL, 0,
-                         minmapsize, NULL, NULL, NULL);
+                         minmapsize, quietmmap, NULL, NULL, NULL);
       out->size=out->dsize[0]=0;
       free(out->array);
       out->array=NULL;
@@ -600,6 +710,7 @@ gal_options_parse_csv_strings_raw(char *string, char *filename, size_t lineno)
   /* The nature of the arrays/numbers read here is very small, so since
      `p->cp.minmapsize' might not have been read yet, we will set it to -1
      (largest size_t number), so the values are kept in memory. */
+  int quietmmap=1;
   size_t minmapsize=-1;
 
 
@@ -640,7 +751,7 @@ gal_options_parse_csv_strings_raw(char *string, char *filename, size_t lineno)
     {
       i=num=gal_list_str_number(list);
       out=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &num, NULL, 0,
-                         minmapsize, NULL, NULL, NULL);
+                         minmapsize, quietmmap, NULL, NULL, NULL);
       for(tstrll=list;tstrll!=NULL;tstrll=tstrll->next)
         ((char **)(out->array))[--i]=tstrll->v;
     }
@@ -651,7 +762,7 @@ gal_options_parse_csv_strings_raw(char *string, char *filename, size_t lineno)
          we'll allocate space for one element, then free it. */
       i=1;
       out=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &i, NULL, 0,
-                         minmapsize, NULL, NULL, NULL);
+                         minmapsize, quietmmap, NULL, NULL, NULL);
       out->size=out->dsize[0]=0;
       free(out->array);
       out->array=NULL;
@@ -810,7 +921,6 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
                           "values to the `--%s' option must be positive",
                           arg, v[i], option->name);
 
-
           if(ceil(v[i]) != v[i])
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "a given "
                           "value in `%s' (%g) is not an integer. The "
@@ -971,6 +1081,100 @@ gal_options_read_sigma_clip(struct argp_option *option, char *arg,
 
 
 
+/* Parse name and (float64) values:  name,value1,value2,value3,...
+
+   The output is a `gal_data_t', where the `name' is the given name and the
+   values are in its array (of `float64' type).
+ */
+void *
+gal_options_parse_name_and_values(struct argp_option *option, char *arg,
+                                  char *filename, size_t lineno, void *junk)
+{
+  size_t i, nc;
+  double *darray;
+  char *c, *name, *values;
+  gal_data_t *tmp, *existing, *dataset;
+  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
+
+  /* We want to print the stored values. */
+  if(lineno==-1)
+    {
+      /* Set the value pointer to `dataset'. */
+      existing=*(gal_data_t **)(option->value);
+      darray = existing->array;
+
+      /* First write the name. */
+      nc=0;
+      nc += sprintf(sstr+nc, "%s,", existing->name);
+
+      /* Write the values into a string. */
+      for(i=0;i<existing->size;++i)
+        {
+          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
+            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
+                  "can address the problem. The number of necessary "
+                  "characters in the statically allocated string has become "
+                  "too close to %d", __func__, PACKAGE_BUGREPORT,
+                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
+          nc += sprintf(sstr+nc, "%g,", darray[i]);
+        }
+      sstr[nc-1]='\0';
+
+      /* Copy the string into a dynamically allocated space, because it
+         will be freed later.*/
+      gal_checkset_allocate_copy(sstr, &str);
+      return str;
+    }
+  else
+    {
+      /* Parse until the comma or the end of the string.*/
+      c=arg; while(*c!='\0' && *c!=',') ++c;
+      values = (*c=='\0') ? NULL : c+1;
+
+      /* Name of the dataset (note that `c' is already pointing the end of
+         the `name' and `values' points to the next character). So we can
+         safely set `c' to `\0' to have the `name'. */
+      *c='\0';
+      gal_checkset_allocate_copy(arg, &name);
+
+      /* Read the values and write the name. */
+      dataset=gal_options_parse_list_of_numbers(values, filename, lineno);
+
+      /* If there actually was a string of numbers, then do the rest. */
+      if(dataset)
+        {
+          dataset->name=name;
+
+          /* Add the given dataset to the end of an existing dataset. */
+          existing = *(gal_data_t **)(option->value);
+          if(existing)
+            {
+              for(tmp=existing;tmp!=NULL;tmp=tmp->next)
+                if(tmp->next==NULL) { tmp->next=dataset; break; }
+            }
+          else
+            *(gal_data_t **)(option->value) = dataset;
+
+          /* For a check.
+             printf("arg: %s\n", arg);
+             darray=dataset->array;
+             for(i=0;i<dataset->size;++i) printf("%f\n", darray[i]);
+             exit(0);
+          */
+        }
+      else
+        error(EXIT_FAILURE, 0, "`--%s' requires a string of numbers "
+              "(separated by `,' or `:') following its first argument, "
+              "please run with `--help' for more information",
+              option->name);
+
+      /* Our job is done, return NULL. */
+      return NULL;
+    }
+}
+
+
+
 
 
 
@@ -1020,7 +1224,7 @@ options_sanity_check(struct argp_option *option, char *arg,
 
   /* Put the option value into a data structure. */
   value=gal_data_alloc(option->value, option->type, 1, &dsize, NULL,
-                       0, -1, NULL, NULL, NULL);
+                       0, -1, 1, NULL, NULL, NULL);
 
   /* Set the operator(s) and operands: */
   switch(option->range)
@@ -1029,7 +1233,7 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GT_0:
       message="greater than zero";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       operator1=GAL_ARITHMETIC_OP_GT;
       ref2=NULL;
@@ -1039,7 +1243,7 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GE_0:
       message="greater or equal to zero";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       operator1=GAL_ARITHMETIC_OP_GE;
       ref2=NULL;
@@ -1049,9 +1253,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_0_OR_1:
       message="either 0 or 1";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=1;
 
@@ -1064,9 +1268,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GE_0_LE_1:
       message="between zero and one (inclusive)";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=1;
 
@@ -1079,9 +1283,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GE_0_LT_1:
       message="between zero (inclusive) and one (exclusive)";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=1;
 
@@ -1094,9 +1298,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GT_0_LT_1:
       message="between zero and one (not inclusive)";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=1;
 
@@ -1109,9 +1313,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_GT_0_ODD:
       message="greater than zero and odd";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=2;
 
@@ -1123,9 +1327,9 @@ options_sanity_check(struct argp_option *option, char *arg,
     case GAL_OPTIONS_RANGE_0_OR_ODD:
       message="greater than, or equal to, zero and odd";
       ref1=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       ref2=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &dsize, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
       *(unsigned char *)(ref1->array)=0;
       *(unsigned char *)(ref2->array)=2;
 
@@ -1146,11 +1350,11 @@ options_sanity_check(struct argp_option *option, char *arg,
      `GAL_ARITHMETIC_INPLACE' flags. But we will do this when there are
      multiple checks so from the two check data structures, we only have
      one remaining. */
-  check1=gal_arithmetic(operator1, GAL_ARITHMETIC_NUMOK, value, ref1);
+  check1=gal_arithmetic(operator1, 1, GAL_ARITHMETIC_NUMOK, value, ref1);
   if(ref2)
     {
-      check2=gal_arithmetic(operator2, GAL_ARITHMETIC_NUMOK, value, ref2);
-      check1=gal_arithmetic(multicheckop, mcflag, check1, check2);
+      check2=gal_arithmetic(operator2, 1, GAL_ARITHMETIC_NUMOK, value, ref2);
+      check1=gal_arithmetic(multicheckop, 1, mcflag, check1, check2);
     }
 
 
@@ -1204,6 +1408,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
          unset). */
       option->func(option, arg, filename, lineno, topass);
       if(option->key!=GAL_OPTIONS_KEY_CONFIG) option->set=GAL_OPTIONS_SET;
+
+      /* The `--config' option is printed for `--checkconfig' by its
+         function (`gal_options_call_parse_config_file'), so must be
+         ignored here. */
+      if(cp->checkconfig && option->key!=GAL_OPTIONS_KEY_CONFIG)
+        printf("  %-25s%s\n", option->name, arg?arg:"ACTIVATED");
       return;
     }
 
@@ -1217,7 +1427,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
       else
         {
           /* If the option is already set, ignore the given value. */
-          if(option->set==GAL_OPTIONS_SET) return;
+          if(option->set==GAL_OPTIONS_SET)
+            {
+              if(cp->checkconfig)
+                printf("  %-25s--ALREADY-SET--\n", option->name);
+              return;
+            }
 
           /* Read the string argument into the value. */
           if( gal_type_from_string(&option->value, arg, option->type) )
@@ -1245,7 +1460,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
   else
     {
       /* If the option is already set, ignore the given value. */
-      if(option->set==GAL_OPTIONS_SET) return;
+      if(option->set==GAL_OPTIONS_SET)
+        {
+          if(cp->checkconfig)
+            printf("  %-25s--ALREADY-SET--\n", option->name);
+          return;
+        }
 
       /* Make sure the option has the type set for options with no
          argument. So, give it a value of 1. */
@@ -1261,6 +1481,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
     }
 
 
+  /* If the user wanted to check the value. */
+  if(cp->checkconfig)
+    printf("  %-25s%s\n", option->name,
+           (arg && option->type!=GAL_OPTIONS_NO_ARG_TYPE)?arg:"ACTIVATED");
+
+
   /* Flip the `set' flag to `GAL_OPTIONS_SET'. */
   option->set=GAL_OPTIONS_SET;
 }
@@ -1537,7 +1763,13 @@ options_set_from_name(char *name, char *arg,  struct argp_option *options,
           if( options[i].flags==OPTION_HIDDEN
               || ( options[i].set
                    && !gal_type_is_list(options[i].type ) ) )
-            return 0;
+            {
+              if(cp->checkconfig)
+                printf("  %-25s%s\n", name, ( options[i].flags==OPTION_HIDDEN
+                                              ? "--IGNORED--"
+                                              : "--ALREADY-SET--" ) );
+              return 0;
+            }
 
           /* Read the value into the option and do a sanity check. */
           gal_options_read_check(&options[i], arg, filename, lineno, cp);
@@ -1596,18 +1828,16 @@ options_parse_file(char *filename,  struct gal_options_common_params *cp,
     return;
 
 
-  /* Open the file. If the file doesn't exist, then just ignore the
-     configuration file and return. */
+  /* Open the file. If the file doesn't exist or can't be opened, then just
+     ignore the configuration file and return. */
   errno=0;
   fp=fopen(filename, "r");
-  if(fp==NULL)
-    {
-      if(errno==ENOENT && enoent_abort==0)
-        return;
-      else
-        error(EXIT_FAILURE, errno, "%s: to be read as a configuration file",
-              filename);
-    }
+  if(fp==NULL) return;
+
+
+  /* If necessary, print the configuration file name. */
+  if(cp->checkconfig)
+    printf("%s:\n", filename);
 
 
   /* Allocate the space necessary to keep a copy of each line as we parse
@@ -1661,11 +1891,26 @@ options_parse_file(char *filename,  struct gal_options_common_params *cp,
 /* This function will be used when the `--config' option is called. */
 void *
 gal_options_call_parse_config_file(struct argp_option *option, char *arg,
-                                   char *filename, size_t lineno, void *cp)
+                                   char *filename, size_t lineno, void *c)
 {
+  struct gal_options_common_params *cp=(struct gal_options_common_params *)c;
+
+  /* The `--config' option is a special function when it comes to
+     `--checkconfig': we'll have to write its value before interpretting
+     it. */
+  if(cp->checkconfig)
+    {
+      printf("  %-25s%s\n", option->name, arg);
+      printf("............................\n");
+    }
+
   /* Call the confguration file parser. */
   options_parse_file(arg, cp, 1);
 
+  /* Ending boundary of this file's options. */
+  if(cp->checkconfig)
+      printf("............................\n");
+
   /* Just to avoid compiler warnings, then return, note that all pointers
      are just copies. */
   option=NULL; filename=NULL; lineno=0;
@@ -1770,6 +2015,51 @@ options_reverse_lists_check_mandatory(struct gal_options_common_params *cp,
 
 
 
+void
+gal_options_read_low_level_checks(struct gal_options_common_params *cp)
+{
+  size_t suggested_mmap=10000000;
+
+  /* If `numthreads' is 0, use the number of threads available to the
+     system. */
+  if(cp->numthreads==0)
+    cp->numthreads=gal_threads_number();
+
+  /* If `minmapsize==0' and quiet isn't given, print a warning. */
+  if(cp->minmapsize==0)
+    {
+      fprintf(stderr, "\n\n"
+              "========= WARNING =========\n"
+              "Minimum size to map an allocated space outside of RAM is "
+              "not set, or set to zero. This can greatly slow down the "
+              "processing of a program or cause strange crashes (recall "
+              "that the number of files that can be memory-mapped is "
+              "limited).\n\n"
+
+              "On modern systems (with RAM larger than a giga-byte), it "
+              "should be fine to set it to %zu (10 million bytes or 10Mb) "
+              "with the command below. In this manner, only arrays that "
+              "are larger than this will be memory-mapped and smaller "
+              "arrays (which are much more numerous) will be allocated and "
+              "freed in the RAM.\n\n"
+
+              "     --minmapsize=%zu\n\n"
+
+              "[This warning can be disabled with the `--quiet' (or `-q') "
+              "option.]\n"
+              "===========================\n\n", suggested_mmap,
+              suggested_mmap);
+    }
+
+  /* If the user wanted to check the parsing of configuration files, then
+     the program must stop here. */
+  if(cp->checkconfig) exit(0);
+}
+
+
+
+
+
 /* Read all configuration files and set common options */
 void
 gal_options_read_config_set(struct gal_options_common_params *cp)
@@ -1786,10 +2076,8 @@ gal_options_read_config_set(struct gal_options_common_params *cp)
   /* Abort if any of the mandatory options are not set. */
   gal_options_abort_if_mandatory_missing(cp);
 
-  /* If `numthreads' is 0, use the number of threads available to the
-     system. */
-  if(cp->numthreads==0)
-    cp->numthreads=gal_threads_number();
+  /* Low-level/basic checks before passing control back to program. */
+  gal_options_read_low_level_checks(cp);
 }
 
 
@@ -2061,6 +2349,7 @@ options_print_all(struct gal_options_common_params *cp, char *dirname,
 {
   size_t i;
   FILE *fp;
+  int errnum;
   time_t rawtime;
   char *topicstr, *filename;
   gal_list_i32_t *group=NULL;
@@ -2073,7 +2362,9 @@ options_print_all(struct gal_options_common_params *cp, char *dirname,
   if(dirname)
     {
       /* Make the host directory if it doesn't already exist. */
-      gal_checkset_mkdir(dirname);
+      if( (errnum=gal_checkset_mkdir(dirname)) )
+        error(EXIT_FAILURE, errnum, "making %s for configuration files",
+              dirname);
 
       /* Prepare the full filename: */
       if( asprintf(&filename, "%s/%s.conf", dirname, cp->program_exec)<0 )
