@@ -6,6 +6,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2019 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -49,4 +50,9 @@ if [ ! -f $table    ]; then echo "$table does not exist."; exit 77; fi
 
 # Actual test script
 # ==================
-$execname $table --output=binary-table.fits --tableformat=fits-binary
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $table --output=binary-table.fits \
+                              --tableformat=fits-binary
